{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit gInfo;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  math, PNGImage,
  StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList;

type
  {Pixel memory access}
  pRGBLine = ^TRGBLine;
  TRGBLine = array[Word] of TRGBTriple;
  pRGBALine = ^TRGBALine;
  TRGBALine = array[Word] of TRGBQuad;

  TIDATAccess = class(TChunkIDAT);

  TGammaTester = class(TCustomPanel)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure OnResize(var Message: TWMSize); message WM_SIZE;
    procedure RecreateBitmap;
  private
    Table: array[Byte] of Byte;
    Bitmap: TBitmap;
    vGamma: Double;
    procedure SetGamma(Value: Double);
  public
    property Gamma: Double read vGamma write SetGamma;
  end;

type
  TFrInfo = class(TForm)
    RightPanel: TPanel;
    RightTitle: TLabel;
    DeleteChunk: TSpeedButton;
    ChunkList: TListView;
    Props: TNotebook;
    PaletteArea: TPaintBox;
    PaletteTitle: TPanel;
    IHDR: TTreeView;
    BarHolder: TPanel;
    SizeHolder: TGroupBox;
    CSize: TLabel;
    USize: TLabel;
    Legend1: TShape;
    Legend2: TShape;
    GAMA: TLabel;
    TestGroup: TGroupBox;
    NoGammaLabel: TLabel;
    DefaultGammaLabel: TLabel;
    KeywordLabel: TLabel;
    TextLabel: TLabel;
    Keyword: TComboBox;
    Text: TMemo;
    MonthLabel: TLabel;
    DayLabel: TLabel;
    YearLabel: TLabel;
    HourLabel: TLabel;
    MinLabel: TLabel;
    SecondLabel: TLabel;
    MonthEdit: TEdit;
    DayEdit: TEdit;
    YearEdit: TEdit;
    MonthScroll: TUpDown;
    YearScroll: TUpDown;
    DayScroll: TUpDown;
    HourScroll: TUpDown;
    HourEdit: TEdit;
    MinEdit: TEdit;
    SecondEdit: TEdit;
    MinScroll: TUpDown;
    SecondScroll: TUpDown;
    Images: TImageList;
    HelpButton: TSpeedButton;
    MainIcon: TImage;
    Label3: TLabel;
    Label1: TLabel;
    Edit9: TEdit;
    Label2: TLabel;
    Edit10: TEdit;
    Label9: TLabel;
    Edit1: TEdit;
    Label10: TLabel;
    Edit2: TEdit;
    Label11: TLabel;
    Edit17: TEdit;
    Edit16: TEdit;
    Edit11: TEdit;
    Edit12: TEdit;
    Edit18: TEdit;
    Edit3: TEdit;
    Edit13: TEdit;
    Edit14: TEdit;
    Edit15: TEdit;
    IICloseBtn: TBitBtn;
    IIHelpBtn: TBitBtn;
    Label8: TLabel;
    Label6: TLabel;
    Label5: TLabel;
    Label15: TLabel;
    Label4: TLabel;
    Label7: TLabel;
    Label12: TLabel;
    AddTextChunk: TSpeedButton;
    AddTimeChunk: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaletteAreaPaint(Sender: TObject);
    procedure ChunkListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure TextExit(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure IIHelpBtnClick(Sender: TObject);
    procedure IICloseBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure LoadII;
    procedure AddTextChunkClick(Sender: TObject);
    procedure DeleteChunkClick(Sender: TObject);
    procedure AddTimeChunkClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    PLTEPalette: TBitmap;
    Gamma1, Gamma2: TGammaTester;
  end;

var
  FrInfo: TFrInfo;

implementation

uses gGlobal, gGrip;

{$R *.DFM}

procedure TFrInfo.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  inherited;
  left := ImageInfoFormX;
  top := ImageInfoFormY;

  Props.PageIndex := 0;

  Gamma1 := TGammaTester.Create(TestGroup);
  Gamma2 := TGammaTester.Create(TestGroup);

  with Gamma1 do
  begin
    Parent := TestGroup;
    Visible := True;
    SetBounds(NoGammaLabel.Left, NoGammaLabel.Top + 3 +
      NoGammaLabel.Height,
      TestGroup.Width - (NoGammaLabel.Left * 2), 15);
    Gamma := 1;
  end;

  with Gamma2 do
  begin
    Parent := TestGroup;
    Visible := True;
    SetBounds(DefaultGammaLabel.Left, DefaultGammaLabel.Top + 3 +
      DefaultGammaLabel.Height,
      TestGroup.Width - (DefaultGammaLabel.Left * 2), 15);
  end;

 { Mypie := TWin95PieChart.Create(BarHolder);
  with MyPie do
  begin
    Parent := BarHolder;
    Visible := TRUE;
    Align := alClient;
    Color := clAppWorkspace;
    Depth := 10;
    DoubleBuffered := TRUE;
  end;}

{  LoadPicture := TPicture.Create;
  LoadPicture.OnProgress := ImageProgress;}

  {Create bitmap to show the PNG palette}
  PltePalette := TBitmap.Create;
  with PltePalette do
  begin
    PixelFormat := pf24Bit;
    Width := Props.Width;
    Height := Props.Height;
  end;

  {Turn on double buffering to avoid flickering}
  if ComponentCount > 0 then
    for i := 0 to ComponentCount - 1 do
      if Components[i] is TWINControl then
        TWINControl(Components[i]).DoubleBuffered := TRUE;
end;

procedure TFrInfo.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(12000);
end;

procedure TFrInfo.IIHelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(12000);
end;

procedure TFrInfo.IICloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TFrInfo.FormClose(Sender: TObject; var Action:
  TCloseAction);
begin
  ImageInfoFormX := FrInfo.left;
  ImageInfoFormY := FrInfo.top;
  DoSaver;
  FormControl:=3;
  GRIPForm.FreeForm;  
end;

procedure TFrInfo.FormDestroy(Sender: TObject);
begin
  Gamma1.Free;
  Gamma2.Free;
{  Mypie.free;}
{  LoadPicture.Free;}
  PltePalette.Free;
end;


procedure TFrInfo.FormShow(Sender: TObject);
begin
  LoadII;
end;

procedure TFrInfo.LoadII;
var
  f: file of Byte;
  size: Longint;
{  PNG: TPNGImage;
    i: Integer;   }
begin {display the image info}
  MakeImage;
  Edit9.Text := GRIPFileName;
  Edit10.Text := ImagePath;
  Edit1.Text := IntToStr(GRIPForm.Image1.Width);
  Edit2.Text := IntToStr(GRIPForm.Image1.Height);
  Edit11.Text := IntToStr(pImageInfo^.bmiHeader.biClrUsed);
  Edit12.Text := IntToStr(pImageInfo^.bmiHeader.biClrImportant);
 {119776} Edit13.Text := IntToStr(pImageInfo^.bmiHeader.biSizeImage);
  {40}{44} Edit14.Text := IntToStr(sizeof(TBitmapInfo));
  {40} Edit15.Text := IntToStr(pImageInfo^.bmiHeader.biSize);
 {24} Edit17.Text := IntToStr(pImageInfo^.bmiHeader.biBitCount);
 {1} Edit16.Text := IntToStr(pImageInfo^.bmiHeader.biPlanes);
 {0} Edit18.Text := IntToStr(pImageInfo^.bmiHeader.biCompression);
  FreeImage;
  AssignFile(f, ImagePath+GRIPFileName);
  Reset(f);
  size := FileSize(f);
  CloseFile(f);
  Edit3.Text := IntToStr(size);

(*
   Props.PageIndex := 0;
  {Set the chunk listbox}
  ChunkList.Items.Clear;
  {Do the work if it is a PNG image}
  if RasterForm.Image1.Picture.Graphic is TPNGImage then
  begin
    {Copy pointer}
    PNG := TPNGImage(RasterForm.Image1.Picture.Graphic);
    if PNG.Chunks.Count > 0 then
    FOR i := 0 TO PNG.Chunks.Count - 1 DO
      with ChunkList.Items.Add do
      begin
        Caption := PNG.Chunks[I].ChunkType;
        subitems.Add(Format('%d bytes', [PNG.Chunks[i].Size]));
      end;
  end;
*)
end;





{---------------------------------------------------------------------}

procedure TFrInfo.DeleteChunkClick(Sender: TObject);
begin
  GRIPForm.Image1.Picture.SaveToFile(ImagePath + GRIPFileName);
end;

procedure TFrInfo.AddTimeChunkClick(Sender: TObject);
{var
  I: Integer;
  PNG: TPNGImage;}
begin
{  PNG := TPNGImage(GRIPForm.Image1.Picture.Graphic);
  PNG.Chunks.Move(PNG.Chunks.AddClass(TChunkTIME).Index,
    PNG.Chunks.Count - 2);
  ChunkList.Items.Clear;
  for i := 0 to PNG.Chunks.Count - 1 do
    with ChunkList.Items.Add do
    begin
      Caption := PNG.Chunks[I].ChunkType;
      subitems.Add(Format('%d bytes', [PNG.Chunks[i].Size]));
    end;}
end;
{---------------------------------------------------------------------}

procedure TFrInfo.AddTextChunkClick(Sender: TObject);
{var
  I: Integer;
  PNG: TPNG5Image;}
begin
{  PNG := TPNG5Image(GRIPForm.Image1.Picture.Graphic);
  PNG.Chunks.Move(PNG.Chunks.AddClass(TChunkTEXT).Index,
    PNG.Chunks.Count - 2);
  ChunkList.Items.Clear;
  for i := 0 to PNG.Chunks.Count - 1 do
    with ChunkList.Items.Add do
    begin
      Caption := PNG.Chunks[I].ChunkType;
      subitems.Add(Format('%d bytes', [PNG.Chunks[i].Size]));
    end;           }
end;
{When the Text TMEMO for editing the tEXt chunk}
{is changed, change the chunk}

procedure TFrInfo.TextExit(Sender: TObject);
{var
  TEXTC: TChunkText;
  ZTXTC: TChunkZTXT;}
begin(*
  if
    TPNGIMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[ChunkList.Selected.Index] is
    TChunkText then
  begin
    {Copy pointer to the chunk}
    TEXTC :=
      TChunkTEXT(TPNG5IMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]);
    {Change the value}
    TEXTC.Keyword := Keyword.Text;
    TEXTC.Text := Text.Lines.Text;
    ChunkList.Selected.SubItems[0] := inttostr(TEXTC.Size) +
      ' bytes';
  end
  else
  begin
    {Copy pointer to the chunk}
    ZTXTC :=
      TChunkZTXT(TPNGIMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]);
    {Change the value}
    ZTXTC.Keyword := Keyword.Text;
    ZTXTC.Text := Text.Lines.Text;
    ChunkList.Selected.SubItems[0] := inttostr(ZTXTC.Size) +
      ' bytes';
  end;
 *)
end;
{---------------------------------------------------------------------}
{---------------------------------------------------------------------}

{When changing ocurrs (like selection change)}

procedure TFrInfo.ChunkListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
{label
  finished;
var
  IDATC: TChunkIDAT;
  PalEntries: WORD;
  IHDRC: TChunkIHDR;
  TEXTC: TChunkText;
  TIMEC: TChunkTime;
  ZTXTC: TChunkZTXT;
  i, j, x: Integer;
  MaxPal: TMaxLogPalette;
  Pal: HPalette;
  Value1,
    Value2,
    Value3,
    Value4: Word;
  DateTime: TDateTime;}
begin(*
  {Only selection matters}
  if not Assigned(ChunkList.Selected) then exit;

  if (ChunkList.Selected.Caption = 'tIME') then
  begin
    Props.PageIndex := 6; {Image1.Picture.}
    TIMEC :=
      TChunkTIME(TPNG5IMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]);
    DateTime := TIMEC.DateTime;
    {Get the values}
    {time}
    DecodeTime(DateTime, Value1, Value2, Value3, Value4);
    HourScroll.Position := Value1;
    MinScroll.Position := Value2;
    SecondScroll.Position := Value3;
    {date}
    DecodeDate(DateTime, Value1, Value2, Value3);
    YearScroll.Position := Value1;
    MonthScroll.Position := Value2;
    DayScroll.Position := Value3;
  end
  else if (ChunkList.Selected.Caption = 'zTXt') then
  begin
    Props.PageIndex := 5;
    ZTXTC :=
      TChunkZTXT(TPNG5IMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]);
    {Set the text values}
    Keyword.Text := ZTXTC.Keyword;
    Text.Text := ZTXTC.Text;
  end
  {If the current selected item is a tEXt chunk}
  else if (ChunkList.Selected.Caption = 'tEXt') then
  begin
    Props.PageIndex := 5 {7};
    TEXTC :=
      TChunkTEXT(TPNG5IMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]);
{Title
Author
Description
Copyright
Creation Time
Software
Disclaimer
Warning
Source
Comment}
    {Set the text values}
    Keyword.Text := TEXTC.Keyword;
    Text.Text := TEXTC.Text;
  end
  {If the current selected item is a gama chunk}
  else if ChunkList.Selected.Caption = 'gAMA' then
  begin
    Props.PageIndex := 4;
    Gama.Caption := 'Gamma intensity: ' + floattostr(
      TChunkGAMA(TPNG5IMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]).Value / 100000);
    Gamma2.Gamma :=
      TChunkGAMA(TPNG5IMAGE(GRIPForm.Image1.Picture.Graphic).Chunks[
      ChunkList.Selected.Index]).Value / 100000;
  end
  {If the current selected item is a palette}
  else if ChunkList.Selected.Caption = 'IDAT' then
  begin
    {Select the IDAT page}
    Props.PageIndex := 3;
    IDATC :=
      TChunkIDAT(TPNG5Image(GRIPForm.Image1.Picture.Graphic).Chunks[ChunkList.Selected.Index]);
{    MyPie.Maximum := 100;
    MyPie.Value := MulDiv(100, IDATC.Size,
      ((TIDATAccess(IDATC).GetBufferWidth + 1) *  LoadPicture.Height));}

    CSize.Caption := '    Compressed: ' +
      inttostr({MyPie.Value} MulDiv(100, IDATC.Size,
      ((TIDATAccess(IDATC).GetBufferWidth + 1) *
      GRIPForm.Image1.Picture.Height)))
      + '%'#13#10 + ChunkList.Selected.subitems[0];
    USize.Caption := '    Uncompressed: 100%'#13#10 +
      inttostr((TIDATAccess(IDATC).GetBufferWidth + 1) *
      GRIPForm.Image1.Picture.Height) + ' bytes';
  end
  {If the current selected item is a palette}
  else if ChunkList.Selected.Caption = 'PLTE' then
  begin
    {Select the palette area}
    Props.PageIndex := 1;
    {Get palette info}
    Pal :=
      TChunkPLTE(TPNG5Image(GRIPForm.Image1.Picture.Graphic).Chunks[ChunkList.Selected.Index]).Palette;
    GetObject(Pal, SIZEOF(WORD), @PalEntries);
    GetPaletteEntries(Pal, 0, PalEntries, MaxPal.palpalentry);
    PaletteTitle.Caption := 'Palette entries: ' +
      inttostr(palentries);

    {Draw the pallete}
    x := 0; j := 0;
    PltePalette.Canvas.Brush.Color := clWhite;
    PltePalette.Canvas.FillRect(Rect(0, 0, PltePalette.Width,
      PltePalette.Height));

    repeat
      i := 0;
      repeat
        with MaxPal.palPalEntry[x] do
          PltePalette.Canvas.Brush.Color := RGB(peRed, peGreen,
            peBlue);
        PltePalette.Canvas.Rectangle(i, j, i + 10, j + 10);
        inc(x);
        inc(i, 10);
        if x > PalEntries - 1 then
          goto FInished;
      until i >= PltePalette.Width;
      inc(j, 10);
    until j >= PltePalette.Height;

    finished:
    {Repaint the palette area}
    PaletteAreaPaint(Sender);
  end
  else if ChunkList.Selected.Caption = 'IHDR' then
  begin
    {Select IHDR page}
    Props.PageIndex := 2;
    {Expand all nodes and remove "checks"}
    for i := 0 to IHDR.Items.Count - 1 do
    begin
      IHDR.Items[i].Expand(TRUE);
      if (IHDR.Items[i].ImageIndex = 0) and
        (IHDR.Items[i].Owner.Count > 1) then
      begin
        IHDR.Items[i].ImageIndex := -1;
        IHDR.Items[i].SelectedIndex := -1;
      end;

    end;

    {Pointer to the IHDR chunk}
    IHDRC :=
      TChunkIHDR(TPNG5Image(GRIPForm.Image1.Picture.Graphic).Chunks[ChunkList.Selected.Index]);

    {Compression}
    IHDR.Items[13].ImageIndex := 0;
    IHDR.Items[13].SelectedIndex := 0;
    {Filter set}
    IHDR.Items[15].ImageIndex := 0;
    IHDR.Items[15].SelectedIndex := 0;

    {Filter set}
    IHDR.Items[19].Text := 'Width: ' + inttostr(IHDRC.Width);
    {Filter set}
    IHDR.Items[20].Text := 'Height: ' + inttostr(IHDRC.Height);

    {Interlacing}
    if IHDRC.Interlaced = 1 then
    begin
      IHDR.Items[17].ImageIndex := 0;
      IHDR.Items[17].SelectedIndex := 0;
    end
    else
    begin
      IHDR.Items[18].ImageIndex := 0;
      IHDR.Items[18].SelectedIndex := 0;
    end;

    {Bit depth}
    case IHDRC.BitDepth of
      1: begin
          IHDR.Items[7].ImageIndex := 0;
          IHDR.Items[7].SelectedIndex := 0;
        end;
      2: begin
          IHDR.Items[8].ImageIndex := 0;
          IHDR.Items[8].SelectedIndex := 0;
        end;
      4: begin
          IHDR.Items[9].ImageIndex := 0;
          IHDR.Items[9].SelectedIndex := 0;
        end;
      8: begin
          IHDR.Items[10].ImageIndex := 0;
          IHDR.Items[10].SelectedIndex := 0;
        end;
      16: begin
          IHDR.Items[11].ImageIndex := 0;
          IHDR.Items[11].SelectedIndex := 0;
        end;
    end;

    {Set the color type}
    case IHDRC.ColorType of
      0: begin
          IHDR.Items[1].ImageIndex := 0;
          IHDR.Items[1].SelectedIndex := 0;
        end;
      2: begin
          IHDR.Items[2].ImageIndex := 0;
          IHDR.Items[2].SelectedIndex := 0;
        end;
      3: begin
          IHDR.Items[3].ImageIndex := 0;
          IHDR.Items[3].SelectedIndex := 0;
        end;
      4: begin
          IHDR.Items[4].ImageIndex := 0;
          IHDR.Items[4].SelectedIndex := 0;
        end;
      6: begin
          IHDR.Items[5].ImageIndex := 0;
          IHDR.Items[5].SelectedIndex := 0;
        end;
    end;

    IHDR.ScrollBy(0, -1000);
  end
  else
    Props.PageIndex := 0;*)
end;
{---------------------------------------------------------------------}
{---------------------------------------------------------------------}
{When painting the area that shows the image palette}

procedure TFrInfo.PaletteAreaPaint(Sender: TObject);
begin
  if Assigned(PltePalette) then
    PaletteArea.Canvas.Draw(0, 0, PltePalette);
end;



{---------------------------------------------------------------------}
{---------------------------------------------------------------------}
{ TGammaTester }

{When the object is being created}

constructor TGammaTester.Create(AOwner: TComponent);
begin
  inherited;

  DoubleBuffered := TRUE;

  {Create the backbuffer bitmap}
  Bitmap := TBitmap.Create;

  {Set 24bits format}
  with Bitmap do
    PixelFormat := pf24bit;

end;

{When the object is being destroyed}

destructor TGammaTester.Destroy;
begin
  {Free the backbuffer bitmap}
  Bitmap.Free;
  inherited;
end;

{When the control is being resized}

procedure TGammaTester.OnResize(var Message: TWMSize);
begin
  inherited;

  RecreateBitmap;
end;


{When the control is being painted}

procedure TGammaTester.Paint;
begin
  inherited;
  Canvas.Draw(0, 0, Bitmap);
end;

{Redraw the gamma bitmap}

procedure TGammaTester.RecreateBitmap;
var
  Line: pRGBLine;
  x, y: Integer;
begin
  {Set the bitmap size}
  Bitmap.Width := Width;
  Bitmap.Height := Height;

  {Paint each line}
  for y := 0 to Bitmap.Height - 1 do
  begin
    Line := Bitmap.ScanLine[y];
    for x := 0 to Bitmap.Width - 1 do
    begin
      {Set the line color}
      with Line^[x] do
      begin
        rgbtRed := Table[MulDiv(255, x, Bitmap.Width)];
        rgbtGreen := rgbtRed;
        rgbtBlue := rgbtRed;
      end;
    end;
  end;

  {Repaint}
  Paint;
end;

{When the property gamma is being set}

procedure TGammaTester.SetGamma(Value: Double);
var
  I: Integer;
begin
  for I := 0 to 255 do
    Table[I] := Round(Power((I / 255), 1 / (Value * 2.2)) * 255);

  vGamma := Value;
  RecreateBitmap;
end;







end.
