{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit gMacro;

interface

uses
  Sysutils, Windows, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Classes, Buttons;
type
  TMacroForm = class(TForm)
    MacroHelpBtn: TSpeedButton;
    MacroMachine: TGroupBox;
    RecordBtn: TSpeedButton;
    LoadBtn: TSpeedButton;
    EditBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    PlayBtn: TSpeedButton;
    PrintBtn: TSpeedButton;
    CopyBtn: TSpeedButton;
    MacroRE: TRichEdit;
    PasteBtn: TSpeedButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    ClearBtn: TSpeedButton;
    PauseBtn: TSpeedButton;
    PreviewCloseBtn: TSpeedButton;
    procedure SpeedButtonToolClick(Sender: TObject);
    procedure MacroHelpBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure PreviewCloseBtnClick(Sender: TObject);
  private
    { Private declarations }
    procedure SetRPO;
    procedure PlaytheMacro;
  public
    { Public declarations }
  end;

var
  MacroForm: TMacroForm;

implementation
uses gGlobal, gGrip, gHistogram, gUISOHO, gLinear, gUILinear,
  gUUnclass,
  gPixelAlgebra, gUPixAl, gUTexture, gUIPFilter, gFEditor,
  gClassifier;
{$R *.DFM}

procedure TMacroForm.FormCreate(Sender: TObject);
begin
  left := MacroFormX;
  top := MacroFormY;
  SaveBtn.Enabled := False;
end;

procedure TMacroForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  LoggingToMacro := FALSE;
  PlayingMacro := FALSE;
  CanClose := True;
end;

procedure TMacroForm.FormClose(Sender: TObject; var Action:
  TCloseAction);
begin
  MacroFormX := MacroForm.left;
  MacroFormY := MacroForm.top;
  DoSaver;
end;

procedure TMacroForm.PreviewCloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TMacroForm.MacroHelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(17500);
end;


procedure TMacroForm.SpeedButtonToolClick(Sender: TObject);
begin
  case (Sender as TSpeedButton).Tag of
    0: begin {Clear}
        MacroRE.Clear;
        LoggingToMacro := FALSE;
        PlayingMacro := FALSE;
        MacroRE.Lines[1] := 'Raster Macro File';
        MacroRE.Lines[2] := GRIPFileName;
        MacroRE.Lines[3] := ImagePath;
         {Set the Program Processing Options}
        SetRPO;
      end;
    1: begin {Record}
        LoggingToMacro := True;
        PlayingMacro := FALSE;
        MacroRE.Lines.Add('Raster Macro File');
        MacroRE.Lines.Add(GRIPFileName);
        MacroRE.Lines.Add(ImagePath);
         {Set the Program Processing Options}
        SetRPO;
        SaveBtn.Enabled := True;
      end;
    2: begin {Load}
        LoggingToMacro := FALSE;
        PlayingMacro := FALSE;
        {Open files to select a # 1  band}
        OpenDialog1.Filter := 'Macro files (eve)|*.eve';
        OpenDialog1.FileName := ChangeFileExt(GRIPFileName, '.eve');
        OpenDialog1.InitialDir := ImagePath;
        if OpenDialog1.execute then begin
          MacroRE.Clear;
          MacroMachine.Caption := OpenDialog1.filename;
          MacroForm.Caption := 'Macronizing: '
            + ExtractFileName(OpenDialog1.filename);
          MacroRE.Lines.LoadFromFile(OpenDialog1.filename);
          SaveBtn.Enabled := True;
        end; end;
    3: begin {Edit}
        LoggingToMacro := FALSE;
        PlayingMacro := FALSE;
        SaveBtn.Enabled := True;
      end;
    4: begin {Save}
        LoggingToMacro := FALSE;
        PlayingMacro := FALSE;
        SaveDialog1.Filter := 'Macro files (eve)|*.eve';
        SaveDialog1.FileName := ChangeFileExt(GRIPFileName, '.eve');
        SaveDialog1.InitialDir := ImagePath;
        if SaveDialog1.execute then begin
          MacroRE.Clear;
          MacroMachine.Caption := OpenDialog1.filename;
          MacroForm.Caption := 'Macronizing: '
            + ExtractFileName(SaveDialog1.filename);
          MacroRE.Lines.SaveToFile(SaveDialog1.filename);
          SaveBtn.Enabled := True;
        end; end;
    5: begin {Play}
        LoggingToMacro := FALSE;
        PlayingMacro := True;
        PlaytheMacro;
      end;
    6: MacroRE.Print(MacroForm.Caption); {Print}
    7: MacroRE.CopytoClipboard; {Copy}
    8: begin
        LoggingToMacro := FALSE;
        PlayingMacro := FALSE;
        MacroRE.PasteFromClipboard; {Paste}
        SaveBtn.Enabled := True;
      end;
    9: {Pause}
      begin
        LoggingToMacro := FALSE;
        PlayingMacro := FALSE;
      end;
  end;
end;

procedure TMacroForm.SetRPO;
begin
  MacroForm.MacroRE.Lines.Add('=0000  Processing Options');
   {V_Absolute, {Absolute Values: -1 become 1}
  if V_Absolute then MacroForm.MacroRE.Lines.Add('Absolute Values:On')
  else MacroForm.MacroRE.Lines.Add('Absolute Values:Off');
  {R_AutoTT Absolute Value Threshold  0.0,255.0 dAT_Max, dAT_Min:Double}
  {iAT_Max Used internally for Integer Spatial Matrix}
  if R_AutoTT then MacroForm.MacroRE.Lines.Add('Set Absolute:On')
  else MacroForm.MacroRE.Lines.Add('Set Absolute:Off');
  MacroForm.MacroRE.Lines.Add(FloatToStr(dAT_Min));
  MacroForm.MacroRE.Lines.Add(FloatToStr(dAT_Max));
  if ClipR then MacroForm.MacroRE.Lines.Add('Clip:On')
  else MacroForm.MacroRE.Lines.Add('Clip:Off');
  if Do_Ex_com then MacroForm.MacroRE.Lines.Add('Do_0..1..255:On')
  else MacroForm.MacroRE.Lines.Add('Do_0..1..255:Off');
  if BelowZero then
    MacroForm.MacroRE.Lines.Add('BelowZero -1..0..1:On')
  else MacroForm.MacroRE.Lines.Add('BelowZero -1..0..1:Off');
  if AutoRange then MacroForm.MacroRE.Lines.Add('AutoRange:On')
  else MacroForm.MacroRE.Lines.Add('AutoRange:Off');
 {S_Threshold Set Threshold  0,255 bST_Min,bST_Max:Byte}
  if S_Threshold then MacroForm.MacroRE.Lines.Add('Set Threshold:On')
  else MacroForm.MacroRE.Lines.Add('Set Threshold:Off');
  MacroForm.MacroRE.Lines.Add(IntToStr(bST_Min));
  MacroForm.MacroRE.Lines.Add(IntToStr(bST_Max));
 { Toggle_D_RGB, Toggle_D_R, Toggle_D_G, Toggle_D_B:Boolean; }
  if Toggle_D_B then MacroForm.MacroRE.Lines.Add('Toggle Blue:On')
  else MacroForm.MacroRE.Lines.Add('Toggle Blue:Off');
  if Toggle_D_G then MacroForm.MacroRE.Lines.Add('Toggle Green:On')
  else MacroForm.MacroRE.Lines.Add('Toggle Green:Off');
  if Toggle_D_R then MacroForm.MacroRE.Lines.Add('Toggle Red:On')
  else MacroForm.MacroRE.Lines.Add('Toggle Red:Off');
  if Toggle_D_RGB then MacroForm.MacroRE.Lines.Add('Toggle RGB:On')
  else MacroForm.MacroRE.Lines.Add('Toggle RGB:Off');
end;

procedure TMacroForm.PlaytheMacro;
var
  ivar1, ivar2, ivar3, ivar4, ivar5, ivar6,
    SSI, I, Codei: Integer;
  dvar1, dvar2, dvar3, dvar4, dvar5, dvar6, dvar7, dvar8, dvar9:
  Double;
  RPO, FP, {FN,} SS: string;
begin
  for I := 0 to MacroRE.Lines.Count - 1 do
  begin
    if Pos('=', MacroRE.Lines[I]) = 1 then
    begin
      SS := copy(MacroRE.Lines[I], 2, 4);
      val(SS, SSI, Codei);
      case SSI of
        0: begin
            RPO := MacroRE.Lines[I + 1];
            if (RPO = 'Absolute Values:On') then
              V_Absolute := True else V_Absolute := False;
          {R_AutoTT Absolute Value Threshold  0.0,255.0 dAT_Max, dAT_Min:Double}
            RPO := MacroRE.Lines[I + 2];
            if (RPO = 'Set Absolute:On') then
              R_AutoTT := True else R_AutoTT := False;
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dAT_Min, Codei);
            RPO := MacroRE.Lines[I + 4];
            val(RPO, dAT_Max, Codei);
            RPO := MacroRE.Lines[I + 5];
            if (RPO = 'Clip:On') then
              ClipR := True else ClipR := False;
            RPO := MacroRE.Lines[I + 6];
            if (RPO = 'Do_0..1..255:On') then
              Do_Ex_com := True else Do_Ex_com := False;
            RPO := MacroRE.Lines[I + 7];
            if (RPO = 'BelowZero -1..0..1:On') then
              BelowZero := True else BelowZero := False;
            RPO := MacroRE.Lines[I + 8];
            if (RPO = 'AutoRange:On') then
              AutoRange := True else AutoRange := False;
        {S_Threshold Set Threshold  0,255 bST_Min,bST_Max:Byte}
            RPO := MacroRE.Lines[I + 9];
            if (RPO = 'Set Threshold:On') then
              S_Threshold := True else S_Threshold := False;
            RPO := MacroRE.Lines[I + 10];
            val(RPO, bST_Min, Codei);
            RPO := MacroRE.Lines[I + 11];
            val(RPO, bST_Max, Codei);
         { Toggle_D_RGB, Toggle_D_R, Toggle_D_G, Toggle_D_B:Boolean; }
            RPO := MacroRE.Lines[I + 12];
            if (RPO = 'Toggle Blue:On') then
              Toggle_D_B := True else Toggle_D_B := False;
            RPO := MacroRE.Lines[I + 13];
            if (RPO = 'Toggle Green:On') then
              Toggle_D_G := True else Toggle_D_G := False;
            RPO := MacroRE.Lines[I + 14];
            if (RPO = 'Toggle Red:On') then
              Toggle_D_R := True else Toggle_D_R := False;
            RPO := MacroRE.Lines[I + 15];
            if (RPO = 'Toggle RGB:On') then
              Toggle_D_RGB := True else Toggle_D_RGB := False;
          end;
        1: UndoImage;
        2: begin
            ImagePath := MacroRE.Lines[I + 1];
            GRIPFileName := MacroRE.Lines[I + 2];
            GRIPForm.SaveImage;
          end;
        3: begin
            ImagePath := MacroRE.Lines[I + 1];
            GRIPFileName := MacroRE.Lines[I + 2];
            FP := ImagePath + GRIPFileName;
            GRIPForm.LoadFromFile(FP);
          end;
        4: begin
            ImagePath := MacroRE.Lines[I + 1];
            GRIPFileName := MacroRE.Lines[I + 2];
            NumberName;
            GRIPForm.SaveImage;
          end;
        5: begin
            ImagePath := MacroRE.Lines[I + 1];
            GRIPFileName := MacroRE.Lines[I + 2];
            GRIPForm.SaveImage;
          end;
        6: begin
          end;
        101: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            HistogramForm.DoH_SCDisplay(ivar1, ivar2);
          end;
{Natural Break Class
Best Fit Class
Steps Class
Area Class
Median Standard D
Mean S Deviation
Arithmatic Increasing
Geometric Increasing
Reciprocal (Decreasing)}

        102: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            HistogramForm.DoH_SOHO(ivar1);
          end;
{Mercury Water
Mars IR
Earth Forest
Venus Soil
Jupiter Urban
Saturn WFU
Uranus Soils
Neptune Combine
Pluto Classed
Red Sun
Blue I O
Green Moon
Krypton
Blue
Blue bIO
Green
Green gMoon
Red
Red rSun
Percent Blue
Percent Green
Percent Red
Percent RGB}

        104: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, dvar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar3, Codei);
            Do_Halley(dvar1, dvar2, dvar3);
          end;
        105: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, dvar1, Codei);
            Do_Asteroids(dvar1);
          end;
        106: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Europa(ivar1);
          end;
        107: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_DoDoClass(ivar1);
          end;
        108: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Class(ivar1);
          end;
        109: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_DoDo(ivar1);
          end;
        110: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Tone(ivar1);
          end;
        111: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_SetOSlice(ivar1);
          end;
        112: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_SetSlice(ivar1);
          end;
        113: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Slice(ivar1);
          end;
        114: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_SetOSIlice(ivar1, ivar2);
          end;
        115: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_SetandSlice(ivar1, ivar2);
          end;
        116: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, dvar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar3, Codei);
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar1, Codei);
            Do_ISlice(dvar1, dvar2, dvar3, ivar1);
          end;

        151: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, ivar3, Codei);
            AContour(ivar1, ivar2, ivar3);
          end;
        152: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, ivar3, Codei);
            Do_Contour(ivar1, ivar2, ivar3);
          end;
        153: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            LinearForm.LinearRG.ItemIndex := ivar2;
            Do_EdgeFollow(ivar1);
          end;
        154: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            LinearForm.LinearRG.ItemIndex := ivar2;
            Do_EdgeFlow(ivar1);
          end;
        155: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            LinearForm.LinearRG.ItemIndex := ivar2;
            ContourInterval(ivar1);
          end;
        156: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            LinearForm.LinearRG.ItemIndex := ivar2;
            DoWireframe(ivar1);
          end;
        159: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, ivar3, Codei);
            Do_Measured_Do(ivar1, ivar2, ivar3);
          end;
        201: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA0.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            PixelAlgebraForm.OpRGMain.ItemIndex := ivar2;
            RPO := MacroRE.Lines[I + 3];
            val(RPO, ivar3, Codei);
            PixelAlgebraForm.BB0.ItemIndex := ivar3;

            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA1.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 5];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA2.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 6];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA3.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 7];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA4.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 8];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA5.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 9];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BA6.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 10];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.OpRGA1.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 11];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.OpRGA2.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 12];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.OpRGA3.ItemIndex := ivar1;

            RPO := MacroRE.Lines[I + 13];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BB1.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 14];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BB2.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 15];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BB3.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 16];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BB4.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 17];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BB5.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 18];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.BB6.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 19];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.OpRGB1.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 20];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.OpRGB2.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 21];
            val(RPO, ivar1, Codei);
            PixelAlgebraForm.OpRGB3.ItemIndex := ivar1;

            PixelAlgebraForm.DOPAPODO;
          end;
        202: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Pixel(ivar1);
          end;
        203: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, dvar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar3, Codei);
            RPO := MacroRE.Lines[I + 4];
            val(RPO, dvar4, Codei);
            RPO := MacroRE.Lines[I + 5];
            val(RPO, dvar5, Codei);
            RPO := MacroRE.Lines[I + 6];
            val(RPO, dvar6, Codei);
            RPO := MacroRE.Lines[I + 7];
            val(RPO, dvar7, Codei);
            RPO := MacroRE.Lines[I + 8];
            val(RPO, dvar8, Codei);
            RPO := MacroRE.Lines[I + 9];
            val(RPO, dvar9, Codei);
            RPO := MacroRE.Lines[I + 10];
            val(RPO, ivar1, Codei);
            Do_PARBMultiply(dvar1, dvar2, dvar3,
              dvar4, dvar5, dvar6,
              dvar7, dvar8, dvar9, ivar1);
          end;
        204: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, dvar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar3, Codei);
            RPO := MacroRE.Lines[I + 10];
            val(RPO, ivar1, Codei);
            Do_PMultiply(dvar1, dvar2, dvar3, ivar1);
          end;
        211: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar3, Codei);
            Do_Gamma(ivar1, ivar2, dvar1, ivar3);
          end;
        212: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 4];
            val(RPO, dvar2, Codei);
            Do_Gamma(ivar1, ivar2, dvar1, dvar2);
          end;
        213: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            RPO := MacroRE.Lines[I + 3];
            val(RPO, dvar1, Codei);
            RPO := MacroRE.Lines[I + 4];
            val(RPO, dvar2, Codei);
            Do_Gamma(ivar1, ivar2, dvar1, dvar2);
          end;
        214: begin
            randomize;
            DoPixel;
          end;
        215: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            DoContrast(ivar1);
          end;
{
Choosy:=FiledFRG.ItemIndex;
FEFilename FilterPath}
        301: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_IntegerFile(FP, ivar2);
          end;
        302: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_RealFile(FP, ivar2);
          end;
        303: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_SobelFile(FP, ivar2);
          end;
        304: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_SoDogFile(FP, ivar2);
          end;
        305: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
         {Range , Threshold foe >255}{Normal ,or Blast it}
         {InRanger , InRangette: Normal is 255,255; 1024,255;  255,0}
            Do_Rotate8SQRTFile(FP, ivar2, 255, 255);
          end;
        306: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_Rotate8MaxFile(FP, ivar2);
          end;
        307: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Ratboy(FP, ivar2);
          end;
        308: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Batgirl(FP, ivar2);
          end;
        309: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_SlopeFile(FP, ivar2, 8);
          end;
        310: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_SlopeFile(FP, ivar2, 6);
          end;
        311: begin
            FilterPath := MacroRE.Lines[I + 1];
            FEFilename := MacroRE.Lines[I + 2];
            RPO := MacroRE.Lines[I + 3]; {Choosy}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar2, Codei);
            FP := FilterPath + FEFilename;
            Do_SlopeFile(FP, ivar2, 8);
          end;
        312: begin
            RPO := MacroRE.Lines[I + 1]; {Choosy... just in case}
            val(RPO, ivar1, Codei);
            FEditForm.FiledFRG.ItemIndex := ivar1;
            Do_SlopeDir;
          end;
        351: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Texture(1, ivar1);
          end;
        352: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Texture(2, ivar1);
          end;
        353: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_Texture(2, ivar1);
            Do_Texture(1, ivar2);
          end;
        354: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_Texture(1, ivar1);
            Do_Texture(2, ivar2);
          end;
        355: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Texture(3, ivar1);
          end;
        356: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Texture(4, ivar1);
          end;
        357: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            Do_Texture(5, ivar1);
          end;
        358: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_Texture(ivar2, ivar1);
          end;
        359: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_ATexture(1, ivar1, ivar2);
          end;
        360: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_ATexture(2, ivar1, ivar2);
          end;
        361: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_ATexture(3, ivar1, ivar2);
          end;
        362: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            Do_ATexture(4, ivar1, ivar2);
          end;
        400: begin
            RPO := MacroRE.Lines[I + 1];
            val(RPO, ivar1, Codei);
            ClassificationForm.CUSeedRG.ItemIndex := ivar1;
            RPO := MacroRE.Lines[I + 2];
            val(RPO, ivar2, Codei);
            ClassificationForm.CUUCRG.ItemIndex := ivar2;
            RPO := MacroRE.Lines[I + 3];
            val(RPO, ivar3, Codei);
            ClassificationForm.CUCREdit.Text := RPO;
            RPO := MacroRE.Lines[I + 4];
            val(RPO, ivar4, Codei);
            ClassificationForm.CUSDEdit.Text := RPO;
            RPO := MacroRE.Lines[I + 5];
            val(RPO, ivar5, Codei);
            ClassificationForm.CURAEdit.Text := RPO;
            RPO := MacroRE.Lines[I + 6];
            val(RPO, ivar6, Codei);
            ClassificationForm.CUMCEdit.Text := RPO;

            if (((ivar1 = 9) and (ivar2 = 2))
              or ((ivar1 = 9) and (ivar2 = 3))
              or (ivar2 > 3)) then
            begin
              RPO := MacroRE.Lines[I + 7];
              TSFileName := RPO;
              ClassificationForm.DoPBoxLoadLoad(TSFileName);
            end;
            if ((ivar1 < 9) and (ivar2 < 4)) then
              ClassificationForm.SetStatsIntoForm(ivar1);
            if ivar2 = 8 then
            begin
              RPO := MacroRE.Lines[I + 8];
              KBFileName := RPO;
            end;
            if ivar2 = 9 then
            begin
              RPO := MacroRE.Lines[I + 8];
              CLayerFileName := RPO;
            end;
            if ivar2 > 9 then
            begin
              RPO := MacroRE.Lines[I + 8];
              KBFileName := RPO;
              RPO := MacroRE.Lines[I + 9];
              CLayerFileName := RPO;
            end;
            if (((ivar1 < 9) and (ivar2 = 0)) or ((ivar1 < 9) and
              (ivar2 = 1))) then
              DoUnClassified(ivar1, ivar2, ivar3, ivar4, ivar5, ivar6)
            else if ((ivar2 = 2) or (ivar2 = 3)) then
              ClassificationForm.DoUnParallel(ivar1, ivar2, ivar3,
                ivar4, ivar5, ivar6)
            else if ivar2 < 8 then
              ClassificationForm.DoClustered(ivar1, ivar2, ivar3,
                ivar4, ivar5, ivar6)
            else if ivar2 = 8 then
              ClassificationForm.DoKBCluster(ivar1, ivar2, ivar3,
                ivar4, ivar5, ivar6)
            else if ivar2 = 9 then
              ClassificationForm.DoLayered(ivar1, ivar2, ivar3,
                ivar4, ivar5, ivar6)
            else if ivar2 > 9 then
              ClassificationForm.DoKBOCluster(ivar1, ivar2, ivar3,
                ivar4, ivar5, ivar6)
            else DoMessages(13);
          end;


        1108: begin

          end;
      end; {Case}
    end;
  end;
end;
{ RasterForm.        FilePath:=ExtractFilePath(Filename);
 FilePath:=MacroRE.Lines[I+1];
  FileName:=MacroRE.Lines[I+2];
  FP:=FilePath+FileName;
    SS:=copy(MacroRE.Lines[I],2,4);
    val(SS,SSI,Codei);


  If LoggingToMacro then
  begin
  MacroForm.MacroRE.Lines.Add('=0104 Saveas');
  MacroForm.MacroRE.Lines.Add(IntToStr(bST_Max));
  MacroForm.MacroRE.Lines.Add(FloatToStr(dAT_Max));
  end;

  If LoggingToMacro then
  begin
  MacroForm.MacroRE.Lines.Add('=0002 Saveas');
  MacroForm.MacroRE.Lines.Add(FilePath);
  MacroForm.MacroRE.Lines.Add(FileName);
  end;
}


(*
If ivar2 <2 then
Begin
           RPO:=MacroRE.Lines[I+7];
           ClassificationForm.PBoxEdit11.Text :=RPO;
           RPO:=MacroRE.Lines[I+8];
           ClassificationForm.PBoxEdit12.Text :=RPO;
           RPO:=MacroRE.Lines[I+9];
           ClassificationForm.PBoxEdit13.Text :=RPO;
           RPO:=MacroRE.Lines[I+10];
           ClassificationForm.PBoxEdit14.Text :=RPO;
           RPO:=MacroRE.Lines[I+11];
           ClassificationForm.PBoxEdit15.Text :=RPO;
           RPO:=MacroRE.Lines[I+12];
           ClassificationForm.PBoxEdit16.Text :=RPO;

           RPO:=MacroRE.Lines[I+13];
           ClassificationForm.PBoxEdit21.Text :=RPO;
           RPO:=MacroRE.Lines[I+14];
           ClassificationForm.PBoxEdit22.Text :=RPO;
           RPO:=MacroRE.Lines[I+15];
           ClassificationForm.PBoxEdit23.Text :=RPO;
           RPO:=MacroRE.Lines[I+16];
           ClassificationForm.PBoxEdit24.Text :=RPO;
           RPO:=MacroRE.Lines[I+17];
           ClassificationForm.PBoxEdit25.Text :=RPO;
           RPO:=MacroRE.Lines[I+18];
           ClassificationForm.PBoxEdit26.Text :=RPO;

           RPO:=MacroRE.Lines[I+19];
           ClassificationForm.PBoxEdit31.Text :=RPO;
           RPO:=MacroRE.Lines[I+20];
           ClassificationForm.PBoxEdit32.Text :=RPO;
           RPO:=MacroRE.Lines[I+21];
           ClassificationForm.PBoxEdit33.Text :=RPO;
           RPO:=MacroRE.Lines[I+22];
           ClassificationForm.PBoxEdit34.Text :=RPO;
           RPO:=MacroRE.Lines[I+23];
           ClassificationForm.PBoxEdit35.Text :=RPO;
           RPO:=MacroRE.Lines[I+24];
           ClassificationForm.PBoxEdit36.Text :=RPO;

           RPO:=MacroRE.Lines[I+25];
           ClassificationForm.PBoxEdit41.Text :=RPO;
           RPO:=MacroRE.Lines[I+26];
           ClassificationForm.PBoxEdit42.Text :=RPO;
           RPO:=MacroRE.Lines[I+27];
           ClassificationForm.PBoxEdit43.Text :=RPO;
           RPO:=MacroRE.Lines[I+28];
           ClassificationForm.PBoxEdit44.Text :=RPO;
           RPO:=MacroRE.Lines[I+29];
           ClassificationForm.PBoxEdit45.Text :=RPO;
           RPO:=MacroRE.Lines[I+30];
           ClassificationForm.PBoxEdit46.Text :=RPO;

           RPO:=MacroRE.Lines[I+31];
           ClassificationForm.PBoxEdit51.Text :=RPO;
           RPO:=MacroRE.Lines[I+32];
           ClassificationForm.PBoxEdit52.Text :=RPO;
           RPO:=MacroRE.Lines[I+33];
           ClassificationForm.PBoxEdit53.Text :=RPO;
           RPO:=MacroRE.Lines[I+34];
           ClassificationForm.PBoxEdit54.Text :=RPO;
           RPO:=MacroRE.Lines[I+35];
           ClassificationForm.PBoxEdit55.Text :=RPO;
           RPO:=MacroRE.Lines[I+36];
           ClassificationForm.PBoxEdit56.Text :=RPO;

           RPO:=MacroRE.Lines[I+37];
           ClassificationForm.PBoxEdit61.Text :=RPO;
           RPO:=MacroRE.Lines[I+38];
           ClassificationForm.PBoxEdit62.Text :=RPO;
           RPO:=MacroRE.Lines[I+39];
           ClassificationForm.PBoxEdit63.Text :=RPO;
           RPO:=MacroRE.Lines[I+40];
           ClassificationForm.PBoxEdit64.Text :=RPO;
           RPO:=MacroRE.Lines[I+41];
           ClassificationForm.PBoxEdit65.Text :=RPO;
           RPO:=MacroRE.Lines[I+42];
           ClassificationForm.PBoxEdit66.Text :=RPO;

           RPO:=MacroRE.Lines[I+43];
           ClassificationForm.PBoxEdit71.Text :=RPO;
           RPO:=MacroRE.Lines[I+44];
           ClassificationForm.PBoxEdit72.Text :=RPO;
           RPO:=MacroRE.Lines[I+45];
           ClassificationForm.PBoxEdit73.Text :=RPO;
           RPO:=MacroRE.Lines[I+46];
           ClassificationForm.PBoxEdit74.Text :=RPO;
           RPO:=MacroRE.Lines[I+47];
           ClassificationForm.PBoxEdit75.Text :=RPO;
           RPO:=MacroRE.Lines[I+48];
           ClassificationForm.PBoxEdit76.Text :=RPO;

           RPO:=MacroRE.Lines[I+49];
           ClassificationForm.PBoxEdit81.Text :=RPO;
           RPO:=MacroRE.Lines[I+50];
           ClassificationForm.PBoxEdit82.Text :=RPO;
           RPO:=MacroRE.Lines[I+51];
           ClassificationForm.PBoxEdit83.Text :=RPO;
           RPO:=MacroRE.Lines[I+52];
           ClassificationForm.PBoxEdit84.Text :=RPO;
           RPO:=MacroRE.Lines[I+53];
           ClassificationForm.PBoxEdit85.Text :=RPO;
           RPO:=MacroRE.Lines[I+54];
           ClassificationForm.PBoxEdit86.Text :=RPO;

           RPO:=MacroRE.Lines[I+55];
           ClassificationForm.PBoxEdit91.Text :=RPO;
           RPO:=MacroRE.Lines[I+56];
           ClassificationForm.PBoxEdit92.Text :=RPO;
           RPO:=MacroRE.Lines[I+57];
           ClassificationForm.PBoxEdit93.Text :=RPO;
           RPO:=MacroRE.Lines[I+58];
           ClassificationForm.PBoxEdit94.Text :=RPO;
           RPO:=MacroRE.Lines[I+59];
           ClassificationForm.PBoxEdit95.Text :=RPO;
           RPO:=MacroRE.Lines[I+60];
           ClassificationForm.PBoxEdit96.Text :=RPO;
End
  else If ivar2 >1 then
Begin

End;
*)


end.
