{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit gPSize;

interface

uses
  Windows, Classes, Graphics, Forms, Controls, Buttons,
  Sysutils, StdCtrls, ExtCtrls, Printers, Spin, Dialogs;

type
  TPrintersizeForm = class(TForm)
    OKBtn: TBitBtn;
    WidthSpinEdit: TSpinEdit;
    HeightSpinEdit: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Panel1: TPanel;
    HelpBtn: TBitBtn;
    AbortBtn: TBitBtn;
    PrinterPagePanel: TPanel;
    Panel2: TPanel;
    TitleEdit: TEdit;
    AuthorEdit: TEdit;
    PrintSizeRG: TRadioGroup;
    PrintStyleCB: TRadioGroup;
    LegendPanel: TPanel;
    TitlePanel: TPanel;
    AuthorPanel: TPanel;
    LegendRG: TRadioGroup;
    LegendBitBtn: TBitBtn;
    PageWidthLabel: TLabel;
    PageHeightLabel: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    FontDialog1: TFontDialog;
    LOFontBtn: TBitBtn;
    CloseBtn: TBitBtn;
    PrinterSizeShape: TShape;
    PALCB: TCheckBox;
    POLCB: TCheckBox;
    procedure FormActivate(Sender: TObject);
    procedure CheckText;
    procedure CheckStyle;
    procedure CheckImage;
    procedure ReActivate;
    procedure HelpBtnClick(Sender: TObject);
    procedure AbortBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PrintStyleCBClick(Sender: TObject);
    procedure PrintSizeRGClick(Sender: TObject);
    procedure LegendBitBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure LOFontBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);

  private
    { Private declarations }
    pwt, pht, bwt, bht: integer;

  public
    { Public declarations }
  end;

var
  PrintersizeForm: TPrintersizeForm;

implementation

{$R *.DFM}

uses gGlobal, gAbout, gGrip, gOMark, gXAnnoLO;

var
  TextOK, ImageOk: Boolean;
  PrintImageWidth, PrintImageHeight,
    TextWidth, TextHeight: Longint;
  TitleStr, AuthorStr: string;

procedure TPrintersizeForm.FormCreate(Sender: TObject);
begin
  left := PrintersizeFormX;
  top := PrintersizeFormY;
  TextOK := False;
  ImageOk := False;
end;

procedure TPrintersizeForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TPrintersizeForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  PrintersizeFormX := PrintersizeForm.left;
  PrintersizeFormY := PrintersizeForm.top;
  DoSaver;
  FormControl:=4;  
  GRIPForm.FreeForm;
end;

procedure TPrintersizeForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(112);
end;

procedure TPrintersizeForm.AbortBtnClick(Sender: TObject);
begin
  if Printer.Printing then
  begin
    Printer.Abort;
    DoMessages(38);
  end;
end;

procedure TPrintersizeForm.FormActivate(Sender: TObject);
{var
Check:Integer;  }
begin
  TitleEdit.Text := GRIPFileName;
//  AuthorEdit.Text := AboutGrip.Licensee.Caption;
  GRIPForm.Image1.Canvas.Font := FontDialog1.Font;

  {Initialize the height spinedit of the printsize dialog box}
  bht := GRIPForm.Image1.Picture.Height;
  PrintersizeForm.HeightSpinEdit.Value := bht;
   {Initialize the width spinedit of the printsize dialog box}
  bwt := GRIPForm.Image1.Picture.Width;
  PrintersizeForm.WidthSpinEdit.Value := bwt;
  pwt := Printer.PageWidth;
  PrintersizeForm.PageWidthLabel.Caption := inttostr(pwt);
  pht := Printer.PageHeight;
  PrintersizeForm.PageHeightLabel.Caption := inttostr(pht);
   {Get the printer orientation: portrait or landscape}
  if (Printer.Orientation = poPortrait) then
    PrintersizeForm.PrintStyleCB.ItemIndex := 0
  else
    PrintersizeForm.PrintStyleCB.ItemIndex := 1;
    { Printer.Orientation := poLandscape;}
  CheckStyle;
end;

procedure TPrintersizeForm.LOFontBtnClick(Sender: TObject);
begin
  if FontDialog1.Execute then
  begin
    GRIPForm.Image1.Canvas.Font := FontDialog1.Font;
  end;
  CheckText;
end;

procedure TPrintersizeForm.LegendBitBtnClick(Sender: TObject);
begin
  CheckText;
end;

procedure TPrintersizeForm.CheckText;
var
  TitleTextSize, AuthorTextSize: TSize;
begin
  TitleStr := TitleEdit.Text;
  AuthorStr := AuthorEdit.Text;
  TextOK := False;
  TitleTextSize := GRIPForm.Image1.Canvas.TextExtent(TitleStr);
  AuthorTextSize := GRIPForm.Image1.Canvas.TextExtent(AuthorStr);
  TextWidth := TitleTextSize.cx + AuthorTextSize.cx;
  if TitleTextSize.cy > AuthorTextSize.cy then
    TextHeight := TitleTextSize.cy
  else TextHeight := AuthorTextSize.cy;
  case LegendRG.ItemIndex of
    0: {No text} TextOK := True;
    1: {Top} if (pwt > TextWidth) then TextOK := True else
        TextOK := False;
    2: {Bottom} if (pwt > TextWidth) then TextOK := True else
        TextOK := False;
  end; {Case}
  if TextOK then LegendBitBtn.Kind := bkOK else
    LegendBitBtn.Kind := bkNo;
  LegendBitBtn.ModalResult := mrNone;
end;

procedure TPrintersizeForm.PrintStyleCBClick(Sender: TObject);
begin
  CheckStyle;
end;

procedure TPrintersizeForm.CheckStyle;
begin
  if PrintStyleCB.ItemIndex = 0 then
  begin
    Printer.Orientation := poPortrait;
    PrinterPagePanel.Top := 8;
    PrinterPagePanel.Left := 8;
    PrinterPagePanel.Height := round(Printer.PageHeight / 13.77)
      {240};
    PrinterPagePanel.Width := round(Printer.PageWidth / 13.77) {185};
    case LegendRG.ItemIndex of
      0..1: begin
          LegendPanel.Top := 8;
          LegendPanel.Left := ((PrinterPagePanel.Width {185} - 166)
            div 2);
        end else
      begin
        LegendPanel.Top := ((PrinterPagePanel.Height {185} - 40));
        LegendPanel.Left := ((PrinterPagePanel.Width {185} - 166) div
          2);
      end;
    end; {case}
  end else
  begin
    Printer.Orientation := poLandscape;
    PrinterPagePanel.Top := 8;
    PrinterPagePanel.Left := 8;
    PrinterPagePanel.Height := round(Printer.PageHeight / 13.77);
    PrinterPagePanel.Width := round(Printer.PageWidth / 13.77);
    case LegendRG.ItemIndex of
      0..1: begin {none top}
          LegendPanel.Top := 8;
          LegendPanel.Left := ((PrinterPagePanel.Width {185} - 166)
            div 2);
        end else
      begin {bottom}
        LegendPanel.Top := ((PrinterPagePanel.Height {185} - 40));
        LegendPanel.Left := ((PrinterPagePanel.Width {185} - 166) div
          2);
      end;
    end; {case}
  end;
  PrintersizeForm.PageWidthLabel.Caption :=
    inttostr(Printer.PageWidth);
  PrintersizeForm.PageHeightLabel.Caption :=
    inttostr(Printer.PageHeight);
  pwt := (Printer.PageWidth);
  pht := (Printer.PageHeight);
  CheckImage;
end;


procedure TPrintersizeForm.PrintSizeRGClick(Sender: TObject);
begin
  CheckImage;
end;

procedure TPrintersizeForm.CheckImage;
begin
  ImageOk := False;
  case PrintSizeRG.ItemIndex of
    0: begin
        if ((pht > bht) and (pwt > bwt)) then
        begin
          ImageOk := True;
          HeightSpinEdit.Value := bht;
          WidthSpinEdit.Value := bwt;
          PrintSizeRG.Color := clBtnFace;
          PrintImageWidth := bwt;
          PrintImageHeight := bht;
          PrinterSizeShape.Height := round(PrintImageHeight / 13.77);
          PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
          PrinterSizeShape.Top := ((PrinterPagePanel.Height
            - PrinterSizeShape.Height) div 2);
          PrinterSizeShape.Left := ((PrinterPagePanel.Width
            - PrinterSizeShape.Width) div 2);
        end else PrintSizeRG.Color := clWindow;
      end;
    1:
      begin
        if ((pht > (bht * 2)) and (pwt > (bwt * 2))) then
        begin
          ImageOk := True;
          HeightSpinEdit.Value := bht * 2;
          WidthSpinEdit.Value := bwt * 2;
          PrintSizeRG.Color := clBtnFace;
          PrintImageWidth := bwt * 2;
          PrintImageHeight := bht * 2;
          PrinterSizeShape.Height := round(PrintImageHeight / 13.77);
          PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
          PrinterSizeShape.Top := ((PrinterPagePanel.Height
            - PrinterSizeShape.Height) div 2);
          PrinterSizeShape.Left := ((PrinterPagePanel.Width
            - PrinterSizeShape.Width) div 2);
        end else PrintSizeRG.Color := clWindow;
      end;
    2:
      begin
        if ((pht > (bht * 3)) and (pwt > (bwt * 3))) then
        begin
          ImageOk := True;
          HeightSpinEdit.Value := bht * 3;
          WidthSpinEdit.Value := bwt * 3;
          PrintSizeRG.Color := clBtnFace;
          PrintImageWidth := bwt * 3;
          PrintImageHeight := bht * 3;
          PrinterSizeShape.Height := round(PrintImageHeight / 13.77);
          PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
          PrinterSizeShape.Top := ((PrinterPagePanel.Height
            - PrinterSizeShape.Height) div 2);
          PrinterSizeShape.Left := ((PrinterPagePanel.Width
            - PrinterSizeShape.Width) div 2);
        end else PrintSizeRG.Color := clWindow;
      end;
    3:
      begin
        if ((pht > (bht * 4)) and (pwt > (bwt * 4))) then
        begin
          ImageOk := True;
          HeightSpinEdit.Value := bht * 4;
          WidthSpinEdit.Value := bwt * 4;
          PrintSizeRG.Color := clBtnFace;
          PrintImageWidth := bwt * 4;
          PrintImageHeight := bht * 4;
          PrinterSizeShape.Height := round(PrintImageHeight / 13.77);
          PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
          PrinterSizeShape.Top := ((PrinterPagePanel.Height
            - PrinterSizeShape.Height) div 2);
          PrinterSizeShape.Left := ((PrinterPagePanel.Width
            - PrinterSizeShape.Width) div 2);
        end else PrintSizeRG.Color := clWindow;
      end;
    4:
      begin
        if ((pht > (bht * 5)) and (pwt > (bwt * 5))) then
        begin
          ImageOk := True;
          HeightSpinEdit.Value := bht * 5;
          WidthSpinEdit.Value := bwt * 5;
          PrintSizeRG.Color := clBtnFace;
          PrintImageWidth := bwt * 5;
          PrintImageHeight := bht * 5;
          PrinterSizeShape.Height := round(PrintImageHeight / 13.77);
          PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
          PrinterSizeShape.Top := ((PrinterPagePanel.Height
            - PrinterSizeShape.Height) div 2);
          PrinterSizeShape.Left := ((PrinterPagePanel.Width
            - PrinterSizeShape.Width) div 2);
        end else PrintSizeRG.Color := clWindow;
      end;
    5:
      begin
        if ((pht > (bht * 6)) and (pwt > (bwt * 6))) then
        begin
          ImageOk := True;
          HeightSpinEdit.Value := bht * 6;
          WidthSpinEdit.Value := bwt * 6;
          PrintSizeRG.Color := clBtnFace;
          PrintImageWidth := bwt * 6;
          PrintImageHeight := bht * 6;
          PrinterSizeShape.Height := round(PrintImageHeight / 13.77);
          PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
          PrinterSizeShape.Top := ((PrinterPagePanel.Height
            - PrinterSizeShape.Height) div 2);
          PrinterSizeShape.Left := ((PrinterPagePanel.Width
            - PrinterSizeShape.Width) div 2);
        end else PrintSizeRG.Color := clWindow;
      end;
    6:
      begin {Compute Max Size}
        if ((pht > bwt) and (pwt > bht)) then
        begin
          if ((pwt div bwt) > (pht div bht)) then
          begin {height controls}
            HeightSpinEdit.Value := bht * (pht div bht);
            WidthSpinEdit.Value := bwt * (pht div bht);
            ImageOk := True;
            PrintSizeRG.Color := clBtnFace;
            PrintImageWidth := bwt * (pht div bht);
            PrintImageHeight := bht * (pht div bht);
            PrinterSizeShape.Height := round(PrintImageHeight /
              13.77);
            PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
            PrinterSizeShape.Top := ((PrinterPagePanel.Height
              - PrinterSizeShape.Height) div 2);
            PrinterSizeShape.Left := ((PrinterPagePanel.Width
              - PrinterSizeShape.Width) div 2);
          end else
          begin
            HeightSpinEdit.Value := bht * (pwt div bwt);
            WidthSpinEdit.Value := bwt * (pwt div bwt);
            ImageOk := True;
            PrintSizeRG.Color := clBtnFace;
            PrintImageWidth := bwt * (pwt div bwt);
            PrintImageHeight := bht * (pwt div bwt);
            PrinterSizeShape.Height := round(PrintImageHeight /
              13.77);
            PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
            PrinterSizeShape.Top := ((PrinterPagePanel.Height
              - PrinterSizeShape.Height) div 2);
            PrinterSizeShape.Left := ((PrinterPagePanel.Width
              - PrinterSizeShape.Width) div 2);
          end;
        end else {Reduce Image to fit Paper}
        begin
          if ((pwt div bwt) > (pht div bht)) then
          begin {height controls}
            HeightSpinEdit.Value := round(bht * ((pht - 100) / bht));
            WidthSpinEdit.Value := round(bwt * ((pht - 100) / bht));
            ImageOk := True;
            PrintSizeRG.Color := clBtnFace;
            PrintImageWidth := round(bwt * ((pht - 100) / bht));
            PrintImageHeight := round(bht * ((pht - 100) / bht));
            PrinterSizeShape.Height := round(PrintImageHeight /
              13.77);
            PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
            PrinterSizeShape.Top := ((PrinterPagePanel.Height
              - PrinterSizeShape.Height) div 2);
            PrinterSizeShape.Left := ((PrinterPagePanel.Width
              - PrinterSizeShape.Width) div 2);
          end else
          begin
            HeightSpinEdit.Value := round(bht * ((pwt - 100) / bwt));
            WidthSpinEdit.Value := round(bwt * ((pwt - 100) / bwt));
            ImageOk := True;
            PrintSizeRG.Color := clBtnFace;
            PrintImageWidth := round(bwt * ((pwt - 100) / bwt));
            PrintImageHeight := round(bht * ((pwt - 100) / bwt));
            PrinterSizeShape.Height := round(PrintImageHeight /
              13.77);
            PrinterSizeShape.Width := round(PrintImageWidth / 13.77);
            PrinterSizeShape.Top := ((PrinterPagePanel.Height
              - PrinterSizeShape.Height) div 2);
            PrinterSizeShape.Left := ((PrinterPagePanel.Width
              - PrinterSizeShape.Width) div 2);
          end;
        end;
      end; {6}
  end; {case}
end;



procedure TPrintersizeForm.ReActivate;
begin
  ImageOk := False;
  TextOK := False;
  if (LegendRG.ItemIndex > 0) then CheckText else TextOK := True;
  if TextOK then CheckImage;
end;


{change the PrinterPagePanel Top Left Height Width
      Canvas.Draw((PageWidth - Bmp.Width) div 2,
                  (PageHeight - Bmp.Height) div 2,
                  Bmp);
within limits of 256 x 256}
{CenteredCB.Checked
Printer.PageWidth
Printer.PageHeight
PrinterPagePanel.Width
PrinterPagePanel.Height
PrinterPagePanel.Left;
PrinterPagePanel.Top;
PrinterMultiImage.Width
PrinterMultiImage.Height
PrinterMultiImage.Left;
PrinterMultiImage.Top;
LegendPanel
AuthorPanel
TitlePanel
TitleEdit
AuthorEdit
LegendRG}


procedure TPrintersizeForm.OKBtnClick(Sender: TObject);
var
  TitleTextSize, AuthorTextSize: TSize;
  PrintRect: TRect;
begin
  ReActivate;
  if ((TextOK and ImageOk)) then
  begin
    if ((not Printer.Printing)) then
    begin
      with Printer do
      begin
        BeginDoc;
        TitleStr := TitleEdit.Text;
        AuthorStr := AuthorEdit.Text;
        TitleTextSize := GRIPForm.Image1.Canvas.TextExtent(TitleStr);
        AuthorTextSize :=
          GRIPForm.Image1.Canvas.TextExtent(AuthorStr);
        TextWidth := TitleTextSize.cx + AuthorTextSize.cx;
        if TitleTextSize.cy > AuthorTextSize.cy then
          TextHeight := TitleTextSize.cy
        else TextHeight := AuthorTextSize.cy;
        if LegendRG.ItemIndex = 1 then
          Canvas.TextOut(((pwt - TextWidth) div 4), (40 +
            TextHeight), TitleStr);
        if LegendRG.ItemIndex = 2 then
          Canvas.TextOut((pwt - TextWidth) div 4, ((pht - 40) -
            TextHeight), TitleStr);
      {TextOut}
        if LegendRG.ItemIndex = 1 then
          Canvas.TextOut((((pwt - TextWidth) div 2) +
            TitleTextSize.cx),
            (40 + TextHeight), AuthorStr);
        if LegendRG.ItemIndex = 2 then
          Canvas.TextOut((((pwt - TextWidth) div 2) +
            TitleTextSize.cx),
            ((pht - 40) - TextHeight), AuthorStr);
        if ((PALCB.Checked) or (POLCB.Checked)) then MakeImage;
        if (PALCB.Checked) then
        begin {Annotation}
          LabelObjectsForm.PaintAll;
        end;
        if (POLCB.Checked) then
        begin {Objects }
          ObjectMarkForm.PaintAll;
        end;
     {Image}
        PrintRect := Rect(
          (pwt - PrintImageWidth) div 2,
          (pht - PrintImageHeight) div 2,
          (((pwt - PrintImageWidth) div 2) + PrintImageWidth),
          (((pht - PrintImageHeight) div 2) + PrintImageHeight));
        Canvas.StretchDraw(PrintRect,
          GRIPForm.Image1.Picture.Bitmap);
        EndDoc;
      end;
      if ((PALCB.Checked) or (POLCB.Checked)) then UndoImage;
    end else DoMessages(37);
  end else DoMessages(36);
end;



end.
