//glommed from Grapic Ex and efg, AFAIK
unit gUScreenPrint;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TFormPrint = class(TForm)
    ButtonPrintBigImage: TButton;
    ButtonColorSpaceArray: TButton;
    ButtonHistograms: TButton;
    ButtonCancel: TButton;
    ButtonPrintSmallImage: TButton;
    procedure ButtonPrintBigImageClick(Sender: TObject);
    procedure ButtonHistogramsClick(Sender: TObject);
    procedure ButtonCancelClick(Sender: TObject);
    procedure ButtonColorSpaceArrayClick(Sender: TObject);
    procedure ButtonPrintSmallImageClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormPrint: TFormPrint;

implementation
{$R *.DFM}


uses
  gGlobal,
  gUColorLibrary, // TColorPlane
  gUHistogramLibrary, // THistogram
  gUImageProcessingPrimitives, // PrintBitmap
  Printers,
  gHistogram, {ShowImageForm,}
  gUStatisticsLibrary;

procedure TFormPrint.FormCreate(Sender: TObject);
begin
  left := FormPrintX;
  top := FormPrintY;
end;

procedure TFormPrint.FormClose(Sender: TObject; var Action:
  TCloseAction);
begin
  FormPrintX := FormPrint.left;
  FormPrintY := FormPrint.top;
  DoSaver;
end;

function CenterText(s: string): INTEGER;
begin
  RESULT := (Printer.PageWidth - Printer.Canvas.TextWidth(s)) div 2
end {CenterText};


procedure PrintFooterTimeStamp(const LeftMargin: INTEGER);
var
  s: string;
begin
    {Footer}
  Printer.Canvas.Font.Name := 'Arial';
  Printer.Canvas.Brush.Color := clWhite;
  Printer.Canvas.Font.Size := 8;
  s := FormatDateTime('m/d/yy  h:nn', Now);
  Printer.Canvas.TextOut(LeftMargin,
    Printer.PageHeight - Printer.Canvas.TextHeight('X'),s);
end {PrinterFooterTimeStamp};


procedure TFormPrint.ButtonPrintBigImageClick(Sender: TObject);
var
  iFromLeftMargin: INTEGER;
  iPrintedImageWidth: INTEGER;
  jDelta: INTEGER;
  jFromTopOfPage: INTEGER;
  jPrintedImageHeight: INTEGER;
  s: string;

begin
  Screen.Cursor := crHourGlass;
  try
    Printer.Orientation := poLandscape;
    Printer.BeginDoc;
      // Header
    Printer.Canvas.Font.Size := 12;
    Printer.Canvas.Font.Name := 'Arial';
    jDelta := Printer.Canvas.TextHeight('X');
    jFromTopOfPage := 3 * jDelta;
    s := HistogramForm. {LabelFilename.} Caption + ', ' +
      HistogramForm.LabelAttributes.Caption;

    if LENGTH(HistogramForm.LabelColorPlane.Caption) > 0
      then
      AppendStr(s, ', ' + HistogramForm.LabelColorPlane.Caption);
    Printer.Canvas.TextOut(CenterText(s), jFromTopOfPage, s);
    jFromTopOfPage := 5 * jDelta;

      // Image position and size
    iFromLeftMargin := MulDiv(Printer.PageWidth, 12, 100); // 12%
    iPrintedImageWidth := MulDiv(Printer.PageWidth, 80, 100); // 80%
      // maintain aspect ratio of bitmap
    jPrintedImageHeight := MulDiv(HistogramForm.BitmapPrint.Height,
      iPrintedImageWidth,
      HistogramForm.BitmapPrint.Width);

      // Won't fit without adjusting height
    if jPrintedImageHeight + 2 * jFromTopOfpage > Printer.PageHeight
      then begin
      jPrintedImageHeight := Printer.PageHeight - 2 * jFromTopOfPage;
      iPrintedImageWidth := MulDiv(HistogramForm.BitmapPrint.Width,
        jPrintedImageHeight,
        HistogramForm.BitmapPrint.Height);
      iFromLeftMargin := (Printer.PageWidth - iPrintedImageWidth) div
        2
    end;

      // Single pixel outline
    Printer.Canvas.Brush.Color := clBlack;
    Printer.Canvas.Rectangle(iFromLeftMargin - 1, jFromTopOfPage - 1,
      iFromLeftMargin + iPrintedImageWidth + 2,
      jFromTopOfPage + jPrintedImageHeight + 2);

      // Print Image
    PrintBitmap(Printer.Canvas,
      Rect(iFromLeftMargin, jFromTopOfPage,
      iFromLeftMargin + iPrintedImageWidth,
      jFromTopOfPage + jPrintedImageHeight),
      HistogramForm.BitmapPrint);

    PrintFooterTimeStamp(iFromLeftMargin);

    Printer.EndDoc;
  finally
    Screen.Cursor := crDefault
  end;
  Close;
end;


procedure TFormPrint.ButtonHistogramsClick(Sender: TObject);
var
  iFromLeftMargin: INTEGER;
  iPrintedImageWidth: INTEGER;
  jDelta: INTEGER;
  jFromTopOfPage: INTEGER;
  jPrintedImageHeight: INTEGER;
  s: string;
  Statistics: TDescriptiveStatistics;

  procedure PrintHistogram(const ColorPlane: TColorplane);
  var
    BitmapHistogram: TBitmap;
    Histogram: THistogram;
    PrintArea: TRect;
  begin
    jFromTopOfPage := jFromTopOfPage + jDelta;
    s := GetColorPlaneString(ColorPlane);
    Printer.Canvas.Font.Size := 12;
    Printer.Canvas.Font.Name := 'Arial';
    Printer.Canvas.Brush.Color := clWhite;
    Printer.Canvas.TextOut(iFromLeftMargin, jFromTopOfPage, s);
    jFromTopOfPage := jFromTopOfPage + jDelta;

    PrintArea := Rect(iFromLeftMargin, jFromTopOfPage,
      iFromLeftMargin + iPrintedImageWidth,
      jFromTopOfPage + jPrintedImageHeight);

    Statistics := TDescriptiveStatistics.Create;

    BitmapHistogram := TBitmap.Create;
    BitmapHistogram.Width := 256;
    BitmapHistogram.Height := 3 * BitmapHistogram.Width div 4;
    try
      {Clear histogram}
      BitmapHistogram.Canvas.Brush.Color := RGB(240, 240, 240);
      BitmapHistogram.Canvas.FillRect(Rect(0, 0,
        BitmapHistogram.Width, BitmapHistogram.Height));
      Histogram := THistogram.Create;
      try
        GetHistogram(HistogramForm.BitmapBig, ColorPlane, Histogram);
        DrawHistogram(ColorPlane, Histogram, BitmapHistogram.Canvas);
      finally
        Histogram.Free
      end;

      {Single pixel outline}
      Printer.Canvas.Brush.Color := clBlack;
      Printer.Canvas.Rectangle(PrintArea.Left - 1, PrintArea.Top - 1,
        PrintArea.Left + iPrintedImageWidth + 2,
        PrintArea.Top + jPrintedImageHeight + 2);

      {Print Image}
      PrintBitmap(Printer.Canvas, PrintArea, BitmapHistogram);

      GetBitmapStatistics(HistogramForm.BitmapBig, ColorPlane,
        Statistics);
      jFromTopOfPage := jFromTopOfPage + jPrintedImageHeight;

      if ColorPlane <> cpHueHSV
        then begin
        s := Format('Min=%0.f  Max=%0.f  Mean=%.1f  S.D.=%.1f',
          [Statistics.MinValue, Statistics.MaxValue,
          Statistics.MeanValue, Statistics.StandardDeviation]);
        Printer.Canvas.Brush.Color := clWhite;
        Printer.Canvas.Font.Size := 8;
        Printer.Canvas.Font.Name := 'Arial';
        Printer.Canvas.TextOut(iFromLeftMargin, jFromTopOfPage, s)
      end

    finally
      BitmapHistogram.Free;
      Statistics.Free;
    end
  end {PrintHistogram};

  {Copied only to be quick -- someday get rid of this}
  procedure PrintImagePlane(const ColorPlane: TColorplane;
    const Invert: BOOLEAN);
  var
    BitmapProcessed: TBitmap;
    PrintArea: TRect;
  begin
    jFromTopOfPage := jFromTopOfPage + jDelta;
    s := GetColorPlaneString(ColorPlane);
    Printer.Canvas.Brush.Color := clWhite;
    Printer.Canvas.TextOut(iFromLeftMargin, jFromTopOfPage, s);
    jFromTopOfPage := jFromTopOfPage + jDelta;

    PrintArea := Rect(iFromLeftMargin, jFromTopOfPage,
      iFromLeftMargin + iPrintedImageWidth,
      jFromTopOfPage + jPrintedImageHeight);

    BitmapProcessed := TBitmap.Create;
    try
      BitmapProcessed.Height := HistogramForm.BitmapBig.Height;
      BitmapProcessed.Width := HistogramForm.BitmapBig.Width;
      BitmapProcessed.PixelFormat :=
        HistogramForm.BitmapBig.PixelFormat;


      BitmapProcessed := ExtractImagePlane(ColorPlane,
        (ColorPlane = cpRGB),
        Invert,
        HistogramForm.BitmapBig);

      // Single pixel outline
      Printer.Canvas.Brush.Color := clBlack;
      Printer.Canvas.Rectangle(PrintArea.Left - 1, PrintArea.Top - 1,
        PrintArea.Left + iPrintedImageWidth + 2,
        PrintArea.Top + jPrintedImageHeight + 2);

      // Print Image
      PrintBitmap(Printer.Canvas, PrintArea, BitmapProcessed);

      jFromTopOfPage := jFromTopOfPage + jPrintedImageHeight
    finally
      BitmapProcessed.Free
    end;
  end {PrintImagePlane};
begin
  Screen.Cursor := crHourGlass;
  try
    Printer.Orientation := poLandscape;
    Printer.BeginDoc;
      {Header}
    Printer.Canvas.Font.Size := 12;
    Printer.Canvas.Font.Name := 'Arial';
    jDelta := Printer.Canvas.TextHeight('X');
    jFromTopOfPage := 2 * jDelta;
    s := HistogramForm. {LabelFilename.} Caption + ', ' +
      HistogramForm.LabelAttributes.Caption;
    Printer.Canvas.TextOut(CenterText(s), jFromTopOfPage, s);

      {Image position and size}

    iPrintedImageWidth := 25 {%} * Printer.PageWidth div 100;
    jPrintedImageHeight := 3 * iPrintedImageWidth div 4;
      {4:3 aspect ratio}

    iFromLeftMargin := 8 {%} * Printer.PageWidth div 100;
    jFromTopOfPage := 3 * jDelta;
    PrintHistogram(cpRed);
    PrintHistogram(cpGreen);
    PrintHistogram(cpBlue);

    iFromLeftMargin := iFromLeftMargin + 5 * iPrintedImageWidth div
      4;
    jFromTopOfPage := 3 * jDelta;
    PrintHistogram(cpIntensity);
    PrintHistogram(cpLightness);
    PrintImagePlane(cpRGB, FALSE);

    iFromLeftMargin := iFromLeftMargin + 5 * iPrintedImageWidth div
      4;
    jFromTopOfPage := 3 * jDelta;
    PrintHistogram(cpHueHSV);
    PrintHistogram(cpSaturationHSV);
    PrintHistogram(cpValue);

    PrintFooterTimeStamp(8 {%} * Printer.PageWidth div 100);

    Printer.EndDoc;
  finally
    Screen.Cursor := crDefault
  end;
  Close
end;


procedure TFormPrint.ButtonCancelClick(Sender: TObject);
begin
  Close
end;


procedure TFormPrint.ButtonColorSpaceArrayClick(Sender: TObject);
var
  iFromLeftMargin: INTEGER;
  iPrintedImageWidth: INTEGER;
  jDelta: INTEGER;
  jFromTopOfPage: INTEGER;
  jPrintedImageHeight: INTEGER;
  s: string;


  procedure PrintImagePlane(const ColorPlane: TColorplane);
  var
    BitmapProcessed: TBitmap;
    PrintArea: TRect;
  begin
    jFromTopOfPage := jFromTopOfPage + jDelta;
    s := GetColorPlaneString(ColorPlane);
    Printer.Canvas.Brush.Color := clWhite;
    Printer.Canvas.TextOut(iFromLeftMargin, jFromTopOfPage, s);
    jFromTopOfPage := jFromTopOfPage + jDelta;

    PrintArea := Rect(iFromLeftMargin, jFromTopOfPage,
      iFromLeftMargin + iPrintedImageWidth,
      jFromTopOfPage + jPrintedImageHeight);

    BitmapProcessed := TBitmap.Create;
    try
      BitmapProcessed.Height := HistogramForm.BitmapBig.Height;
      BitmapProcessed.Width := HistogramForm.BitmapBig.Width;
      BitmapProcessed.PixelFormat :=
        HistogramForm.BitmapBig.PixelFormat;

      BitmapProcessed := ExtractImagePlane(ColorPlane,
        (ColorPlane = cpRGB) or
        (ColorPlane = cpHueHSV) or
        (ColorPlane = cpHueHLS),
        FALSE,
        HistogramForm.BitmapBig);

      {Single pixel outline}
      Printer.Canvas.Brush.Color := clBlack;
      Printer.Canvas.Rectangle(PrintArea.Left - 1, PrintArea.Top - 1,
        PrintArea.Left + iPrintedImageWidth + 2,
        PrintArea.Top + jPrintedImageHeight + 2);

      {Print Image}
      PrintBitmap(Printer.Canvas, PrintArea, BitmapProcessed);

      jFromTopOfPage := jFromTopOfPage + jPrintedImageHeight
    finally
      BitmapProcessed.Free
    end;
  end {PrintImagePlane};
begin
  Screen.Cursor := crHourGlass;
  try
    Printer.Orientation := poLandscape;
    Printer.BeginDoc;
      {Header}
    Printer.Canvas.Font.Size := 12;
    Printer.Canvas.Font.Name := 'Arial';
    jDelta := Printer.Canvas.TextHeight('X');
    jFromTopOfPage := 2 * jDelta;
    s := HistogramForm. {LabelFilename.} Caption + ', ' +
      HistogramForm.LabelAttributes.Caption;
    Printer.Canvas.TextOut(CenterText(s), jFromTopOfPage, s);

      // Image position and size

    iPrintedImageWidth := 25 {%} * Printer.PageWidth div 100;
      // Make sure image fits, even if distortion is necessary
      // Assumes aspect ratio of 4:3
    jPrintedImageHeight := 3 * iPrintedImageWidth div 4;

    iFromLeftMargin := 8 {%} * Printer.PageWidth div 100;
    jFromTopOfPage := 3 * jDelta;
    PrintImagePlane(cpRed);
    PrintImagePlane(cpGreen);
    PrintImagePlane(cpBlue);

    iFromLeftMargin := iFromLeftMargin + 5 * iPrintedImageWidth div
      4;
    jFromTopOfPage := 3 * jDelta;
    PrintImagePlane(cpIntensity);
    PrintImagePlane(cpLightness);
    PrintImagePlane(cpRGB);

    iFromLeftMargin := iFromLeftMargin + 5 * iPrintedImageWidth div
      4;
    jFromTopOfPage := 3 * jDelta;
    PrintImagePlane(cpHueHSV);
    PrintImagePlane(cpSaturationHSV);
    PrintImagePlane(cpValue);

    PrintFooterTimeStamp(8 {%} * Printer.PageWidth div 100);

    Printer.EndDoc;
  finally
    Screen.Cursor := crDefault
  end;
  Close
end;


procedure TFormPrint.ButtonPrintSmallImageClick(Sender: TObject);
var
  iFromLeftMargin: INTEGER;
  iPrintedImageWidth: INTEGER;
  jDelta: INTEGER;
  jFromTopOfPage: INTEGER;
  jPrintedImageHeight: INTEGER;
  s: string;

begin
  Screen.Cursor := crHourGlass;
  try
    Printer.Orientation := poPortrait;
    Printer.BeginDoc;
      // Header
    Printer.Canvas.Font.Size := 12;
    Printer.Canvas.Font.Name := 'Arial';
    jDelta := Printer.Canvas.TextHeight('X');
    jFromTopOfPage := 3 * jDelta;
    s := HistogramForm. {LabelFilename.} Caption + ', ' +
      HistogramForm.LabelAttributes.Caption;

    if LENGTH(HistogramForm.LabelColorPlane.Caption) > 0
      then
      AppendStr(s, ', ' + HistogramForm.LabelColorPlane.Caption);
    Printer.Canvas.TextOut(CenterText(s), jFromTopOfPage, s);
    jFromTopOfPage := 5 * jDelta;

      // Image position and size
    iFromLeftMargin := 30 {%} * Printer.PageWidth div 100;
    iPrintedImageWidth := 40 {%} * Printer.PageWidth div 100;
      // maintain aspect ratio of bitmap
    jPrintedImageHeight := HistogramForm.BitmapPrint.Height *
      iPrintedImageWidth div
      HistogramForm.BitmapPrint.Width;
      // Single pixel outline
    Printer.Canvas.Brush.Color := clBlack;
    Printer.Canvas.Rectangle(iFromLeftMargin - 1, jFromTopOfPage - 1,
      iFromLeftMargin + iPrintedImageWidth + 2,
      jFromTopOfPage + jPrintedImageHeight + 2);

      // Print Image
    PrintBitmap(Printer.Canvas,
      Rect(iFromLeftMargin, jFromTopOfPage,
      iFromLeftMargin + iPrintedImageWidth,
      jFromTopOfPage + jPrintedImageHeight),
      HistogramForm.BitmapPrint);

    iFromLeftMargin := 8 {%} * Printer.PageWidth div 100;
    PrintFooterTimeStamp(iFromLeftMargin);

    Printer.EndDoc;
  finally
    Screen.Cursor := crDefault
  end;
  Close;
end;



end.
