{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit gXAnnoLO;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, ColorGrd, ComCtrls;

type
  TLabelObjectsForm = class(TForm)
    LOTEdit: TEdit;
    TransparentTextCB: TCheckBox;
    ColorGrid1: TColorGrid;
    LOXEdit: TEdit;
    LOYEdit: TEdit;
    MousedLocationCB: TCheckBox;
    LORAEdit: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Bevel1: TBevel;
    FontDialog1: TFontDialog;
    LOFontBtn: TBitBtn;
    Bevel3: TBevel;
    UpDown1: TUpDown;
    SaveListBtn: TSpeedButton;
    LoadListBtn: TSpeedButton;
    Delete1ListBtn: TSpeedButton;
    OverlayListBtn: TSpeedButton;
    HelpBtn: TSpeedButton;
    UpDown2: TUpDown;
    CurrentListEdit: TEdit;
    Add1ListBtn: TSpeedButton;
    CloseBtn: TSpeedButton;
    LOSaveDialog: TSaveDialog;
    LOOpenDialog: TOpenDialog;
    PlaceBtn: TSpeedButton;
    MakeBtn: TSpeedButton;
    ClearAllBtn: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    RotatedCB: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);


    procedure ColorGrid1Click(Sender: TObject);
    function GetColorGridIndex(Incolor: TColor): Integer;
    procedure LOFontBtnClick(Sender: TObject);

    procedure UpDown2Click(Sender: TObject; Button: TUDBtnType);
    procedure SelectRotation(Rotate: Boolean);

    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure SelectNextListItem(Rotate: Boolean);
    procedure UpDown1ChangingEx(Sender: TObject; var AllowChange:
      Boolean;
      NewValue: Smallint; Direction: TUpDownDirection);

    procedure ShowData(Which: Integer);
    procedure GetData(Which: Integer);
    procedure Add1ListBtnClick(Sender: TObject);
    procedure Delete1ListBtnClick(Sender: TObject);

    procedure LoadListBtnClick(Sender: TObject);
    procedure SaveListBtnClick(Sender: TObject);
    procedure OverlayListBtnClick(Sender: TObject);
    procedure MakeBtnClick(Sender: TObject);
    procedure ClearAllBtnClick(Sender: TObject);

    procedure PlaceBtnClick(Sender: TObject);
    procedure PlaceLabel(X, Y: Integer);
    procedure PaintAll;
    procedure PlaceThisLabel(ThisLabel: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  LabelObjectsForm: TLabelObjectsForm;
  CurrentLabel,
    LabelListCount: Integer;

implementation

uses gGlobal, gGrip;

{$R *.DFM}

procedure TLabelObjectsForm.FormCreate(Sender: TObject);
begin
  left := LabelObjectsFormX;
  top := LabelObjectsFormY;
  LabelListCount := 0;
  CurrentLabel := 0;
  MousedLocationCB.Checked := True;
  MouseLocatext := True;
  TransparentTextCB.Checked := False;
  ObjectLabeling := False;
  ObjectLabelingOn := False;
  SetLength(ObjectLabelArray, LabelListCount + 1);
  GetData(CurrentLabel);
end;

procedure TLabelObjectsForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(14000);
end;

procedure TLabelObjectsForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TLabelObjectsForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  LabelObjectsFormX := LabelObjectsForm.left;
  LabelObjectsFormY := LabelObjectsForm.top;
  DoSaver;
  FormControl:=10;  
  GRIPForm.FreeForm;   
end;

{Choose the Color and place choice into Shadow Edit box}
{clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,
clGray,clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite
clDkGray,clLtGray,}

procedure TLabelObjectsForm.ColorGrid1Click(Sender: TObject);
var ColorChoice: Integer;
begin
  ColorChoice := ColorGrid1.BackgroundIndex;
  case ColorChoice of
    0: LOTEdit.Color := clBlack;
    1: LOTEdit.Color := clMaroon;
    2: LOTEdit.Color := clGreen;
    3: LOTEdit.Color := clOlive;
    4: LOTEdit.Color := clNavy;
    5: LOTEdit.Color := clPurple;
    6: LOTEdit.Color := clTeal;
    7: LOTEdit.Color := clGray;
    8: LOTEdit.Color := clSilver;
    9: LOTEdit.Color := clRed;
    10: LOTEdit.Color := clLime;
    11: LOTEdit.Color := clYellow;
    12: LOTEdit.Color := clBlue;
    13: LOTEdit.Color := clFuchsia;
    14: LOTEdit.Color := clAqua;
    15: LOTEdit.Color := clWhite;
  end;
  ColorChoice := ColorGrid1.ForegroundIndex;
  case ColorChoice of
    0: LOTEdit.Font.Color := clBlack;
    1: LOTEdit.Font.Color := clMaroon;
    2: LOTEdit.Font.Color := clGreen;
    3: LOTEdit.Font.Color := clOlive;
    4: LOTEdit.Font.Color := clNavy;
    5: LOTEdit.Font.Color := clPurple;
    6: LOTEdit.Font.Color := clTeal;
    7: LOTEdit.Font.Color := clGray;
    8: LOTEdit.Font.Color := clSilver;
    9: LOTEdit.Font.Color := clRed;
    10: LOTEdit.Font.Color := clLime;
    11: LOTEdit.Font.Color := clYellow;
    12: LOTEdit.Font.Color := clBlue;
    13: LOTEdit.Font.Color := clFuchsia;
    14: LOTEdit.Font.Color := clAqua;
    15: LOTEdit.Font.Color := clWhite;
  end;
end;

function TLabelObjectsForm.GetColorGridIndex(Incolor: TColor):
Integer;
begin
  case Incolor of
    clBlack: Result := 0;
    clMaroon: Result := 1;
    clGreen: Result := 2;
    clOlive: Result := 3;
    clNavy: Result := 4;
    clPurple: Result := 5;
    clTeal: Result := 6;
    clGray: Result := 7;
    clSilver: Result := 8;
    clRed: Result := 9;
    clLime: Result := 10;
    clYellow: Result := 11;
    clBlue: Result := 12;
    clFuchsia: Result := 13;
    clAqua: Result := 14;
    clWhite: Result := 15;
  else Result := 15;
  end;
end;


procedure TLabelObjectsForm.LOFontBtnClick(Sender: TObject);
begin {Open Font Dialog}
  FontDialog1.Font := LOTEdit.Font;
  if FontDialog1.Execute then
  begin
    LOTEdit.Font := FontDialog1.Font;
  end;
end;


procedure TLabelObjectsForm.UpDown2Click(Sender: TObject;
  Button: TUDBtnType);
begin
  SelectRotation(Button = btNext);
end;

procedure TLabelObjectsForm.SelectRotation(Rotate: Boolean);
begin
{  If ((ObjectLabelArray[CurrentLabel].RotAngle = 359)
  and (Rotate))  then UpDown2.Position:=3
  else
  If ((ObjectLabelArray[CurrentLabel].RotAngle = 0)
  and (not Rotate))  then UpDown2.Position:=355
else}
  begin
{If Rotate then inc(ObjectLabelArray[CurrentLabel].RotAngle)
          else dec(ObjectLabelArray[CurrentLabel].RotAngle);}
    ObjectLabelArray[CurrentLabel].RotAngle := UpDown2.Position;
    Label2.Caption :=
      IntToStr(ObjectLabelArray[CurrentLabel].RotAngle);
    ShowData(CurrentLabel);
    GetData(CurrentLabel);
    PlaceThisLabel(CurrentLabel);
  end;
end;


procedure TLabelObjectsForm.UpDown1Click(Sender: TObject;
  Button: TUDBtnType);
begin
  SelectNextListItem(Button = btNext);
end;

procedure TLabelObjectsForm.SelectNextListItem(Rotate: Boolean);
begin
  if Rotate then inc(CurrentLabel) else
    dec(CurrentLabel);
  ShowData(CurrentLabel);
end;

procedure TLabelObjectsForm.UpDown1ChangingEx(Sender: TObject;
  var AllowChange: Boolean; NewValue: Smallint;
  Direction: TUpDownDirection);
begin
  if (NewValue > LabelListCount) then AllowChange := False
  else if (NewValue < 0) then AllowChange := False
  else AllowChange := True;
  if AllowChange then GetData(CurrentLabel);
{
Write an OnChanging event handler
to conditionally disallow changes
to the value of Position.
Set the AllowChange parameter to False
to prevent a change to the current value of Position.

OnChanging occurs both when
the user clicks the arrow buttons of the up-down control
and when an attempt is made to
change the value of Position programmatically.
To determine which button the user clicked,
use the OnChangingEx event or the OnClick event,
both of which follow the OnChanging event.}
end;

procedure TLabelObjectsForm.Add1ListBtnClick(Sender: TObject);
begin
  {Store current}
  GetData(CurrentLabel);
  {Set next}
  inc(LabelListCount);
  CurrentLabel := LabelListCount;
  SetLength(ObjectLabelArray, LabelListCount + 1);
  GetData(CurrentLabel);
  ShowData(CurrentLabel);
  UpDown1.Position := CurrentLabel;
end;

procedure TLabelObjectsForm.Delete1ListBtnClick(Sender: TObject);
var I: Integer;
begin
  if (LabelListCount > 0) then
  begin
    if CurrentLabel = LabelListCount then
    begin
      dec(CurrentLabel);
      dec(LabelListCount);
      SetLength(ObjectLabelArray, LabelListCount + 1);
    end else
    begin
      for I := CurrentLabel to LabelListCount - 1 do
      begin
        ObjectLabelArray[I] := ObjectLabelArray[I + 1];
      end;
      CurrentLabel := 0;
      dec(LabelListCount);
      SetLength(ObjectLabelArray, LabelListCount + 1);
    end;
    ShowData(CurrentLabel);
    UpDown1.Position := CurrentLabel;
  end;
end;

procedure TLabelObjectsForm.ShowData(Which: Integer);
begin
  LOTEdit.Text := ObjectLabelArray[Which].LOText;
  LORAEdit.Text := IntToStr(ObjectLabelArray[Which].RotAngle);
  LOXEdit.Text := IntToStr(ObjectLabelArray[Which].X);
  LOYEdit.Text := IntToStr(ObjectLabelArray[Which].Y);
  RotatedCB.Checked := ObjectLabelArray[Which].Rotated;
  TransparentTextCB.Checked := ObjectLabelArray[Which].TransBack;
  LOTEdit.Color := ObjectLabelArray[Which].BackColor;
  ColorGrid1.BackGroundIndex := GetColorGridIndex(LOTEdit.Color);
  LOTEdit.Font := ObjectLabelArray[Which].LOFont;
  ColorGrid1.ForeGroundIndex :=
    GetColorGridIndex(LOTEdit.Font.Color);
end;


procedure TLabelObjectsForm.GetData(Which: Integer);
begin
  ObjectLabelArray[Which].LOText := LOTEdit.Text;
  ObjectLabelArray[Which].RotAngle := StrToInt(LORAEdit.Text);
  ObjectLabelArray[Which].X := StrToInt(LOXEdit.Text);
  ObjectLabelArray[Which].Y := StrToInt(LOYEdit.Text);
  ObjectLabelArray[Which].Rotated := RotatedCB.Checked;
  ObjectLabelArray[Which].TransBack := TransparentTextCB.Checked;
  ObjectLabelArray[Which].BackColor := LOTEdit.Color;
  ObjectLabelArray[Which].LOFont := LOTEdit.Font;

end;

procedure TLabelObjectsForm.LoadListBtnClick(Sender: TObject);
var fLoad: file of ObjectLabel;
  Incoming: ObjectLabel;
begin
  LOOpenDialog.InitialDir := ImagePath;
  LOOpenDialog.Filter := 'Raster Attribute File (*.raf)|*.raf';
  LOOpenDialog.FileName := ImagePath + ChangeFileExt(GRIPFileName,
    '.raf');
  if LOOpenDialog.Execute then
    LOFileName := LOOpenDialog.Filename;
{$I-}
  AssignFile(fLoad, LOFileName);
  Reset(fLoad);
  LabelListCount := -1;
  while (not eof(fLoad)) do
  begin
    Read(fLoad, Incoming);
    inc(LabelListCount);
    SetLength(ObjectLabelArray, LabelListCount + 1);
    ObjectLabelArray[LabelListCount] := Incoming;
  end;
  CloseFile(fLoad); {$I+}
  CurrentLabel := 0;
  ShowData(CurrentLabel);
  UpDown1.Position := CurrentLabel;
end;



procedure TLabelObjectsForm.SaveListBtnClick(Sender: TObject);
var
  I: Integer;
  fLoad: file of ObjectLabel;
  OutGoing: ObjectLabel;
begin {Save Training sites}
  LOSaveDialog.InitialDir := ImagePath;
  LOSaveDialog.FileName := ImagePath + ChangeFileExt(GRIPFileName,
    '.raf');
  LOSaveDialog.Filter := 'Raster Object File (*.raf)|*.raf';
  if LOSaveDialog.execute then begin
    LOFileName := LOSaveDialog.Filename;
{$I-}
    AssignFile(fLoad, LOSaveDialog.filename);
    Rewrite(fLoad);
    for I := 0 to LabelListCount do
    begin
      OutGoing := ObjectLabelArray[I];
      Write(fLoad, OutGoing);
    end;
    CloseFile(fLoad); {$I+}
  end;
end;


{Article #16854: Big-endian and little-endian formated integers}
{Article #16790: How can I activate different hints for each cell in a grid}

procedure TLabelObjectsForm.MakeBtnClick(Sender: TObject);
begin
{Form1.Show;}
  MakeImage;
end;

procedure TLabelObjectsForm.ClearAllBtnClick(Sender: TObject);
begin
  UndoImage;
end;

procedure TLabelObjectsForm.OverlayListBtnClick(Sender: TObject);
begin
  PaintAll;
end;

procedure TLabelObjectsForm.PaintAll;
var I: Integer;
begin
  for I := 0 to LabelListCount do
  begin
    PlaceThisLabel(I);
  end;
end;


procedure TLabelObjectsForm.PlaceBtnClick(Sender: TObject);
begin
  GetData(CurrentLabel);
  if MousedLocationCB.Checked then
  begin
    ObjectLabeling := True;
    GRIPForm.Show;
  end else
  begin
    ObjectLabeling := False;
    ObjectLabelingOn := False;
    GRIPForm.Show;
    PlaceLabel(StrToInt(LOXEdit.Text), StrToInt(LOYEdit.Text));
  end;
end;

procedure TLabelObjectsForm.PlaceLabel(X, Y: Integer);
begin
  GetData(CurrentLabel);
  ObjectLabelArray[CurrentLabel].X := X;
  ObjectLabelArray[CurrentLabel].Y := Y;
  PlaceThisLabel(CurrentLabel);
end;
(*
procedure RotTextOut(x, y, Angle, Size: Integer; Txt: String);
var
  HFont,
  OldFont:  Integer;
  DC:       HDC;
  FontName: String;
begin
  if Length(Txt)=0 then exit;
  DC:=Screen.ActiveForm.Canvas.Handle;
  SetBkMode(DC, TRANSPARENT);
  if not(FontChanged) then FontName:=Screen.ActiveForm.Canvas.Font.Name else
    FontName:=Form1.FontDialog1.Font.Name;
  HFont:=CreateFont(-Size, 0, Angle*10, 0, fw_normal,0,0,0,1,4,$10,2,4,PChar(FontName));
  OldFont:=SelectObject(DC, HFont);
  TextOut(DC, x, y, PChar(Txt), Length(Txt));
  SelectObject(DC, OldFont);
  DeleteObject(HFont);
end;

*)
(*
procedure TLabelObjectsForm.PlaceThisLabel(ThisLabel:Integer);
var
  HFont,
  OldFonti:  Integer;
OldBkMode : integer;
FontRecord:TLogFont;
{OldFont:TFont; }
Begin{OK ... PLACE the text}
  OldBkMode :=0;
if Image1.Canvas.Font = nil then
   Image1.Canvas.Font:=LOTEdit.Font;
{  OldFont:=RasterForm.Image1.Canvas.Font;}
  Image1.Canvas.Font:=ObjectLabelArray[ThisLabel].LOFont;
  If ObjectLabelArray[ThisLabel].TransBack then
  OldBkMode :=
  SetBkMode(Image1.Canvas.Handle,0{TRANSPARENT})
  else SetBkColor(Image1.Canvas.Handle,
       ObjectLabelArray[ThisLabel].BackColor);

         GetObject(Image1.Canvas.Font.Handle,
           Sizeof(FontRecord),
           Addr(FontRecord));
{  FontRecord.lfEscapement:=(ObjectLabelArray[ThisLabel].RotAngle *10);}
{  FontRecord.lfOrientation:=(ObjectLabelArray[ThisLabel].RotAngle *10);}
{  RasterForm.Image1.Canvas.Font.Handle:=CreateFontIndirect(FontRecord);}

{  RasterForm.Image1.Canvas.Font:=}
{  HFont:=CreateFont(-Size, 0, Angle*10, 0, fw_normal,0,0,0,1,4,$10,2,4,PChar(FontName));}

HFont:=
CreateFont(
{nHeight} FontRecord.lfHeight,
{nWidth} FontRecord.lfWidth,
{nEscapement} {FontRecord.lfEscapement,}
(ObjectLabelArray[ThisLabel].RotAngle *10),
{nOrientaion}{FontRecord.lfOrientation,}(ObjectLabelArray[ThisLabel].RotAngle *10),
{fnWeight}FontRecord.lfWeight,
{fdwItalic}FontRecord.lfItalic,
{fdwUnderline}FontRecord.lfUnderline,
{fdwStrikeOut}FontRecord.lfStrikeOut,
{fdwCharSet}FontRecord.lfCharSet,
{fdwOutputPrecision}FontRecord.lfOutPrecision,
{fdwClipPrecision}FontRecord.lfClipPrecision,
{fdwQuality}FontRecord.lfQuality,
{fdwPitchAndFamily}FontRecord.lfPitchAndFamily,
{lpszFace}FontRecord.lfFaceName);
  OldFonti:=SelectObject(Image1.Canvas.Font.Handle, HFont);

  Image1.Canvas.TextOut(
        ObjectLabelArray[ThisLabel].X,
        ObjectLabelArray[ThisLabel].Y,
        ObjectLabelArray[ThisLabel].LOText);
  SelectObject(Image1.Canvas.Font.Handle, OldFonti);
  DeleteObject(HFont);
{  RasterForm.Image1.Canvas.Font:=OldFont;}
  If ObjectLabelArray[ThisLabel].TransBack then
  SetBkMode(Image1.Canvas.Handle,
            OldBkMode);
End;
*)
{ObjectLabel = record
LOText:String[250];
RotAngle,X,Y:Integer;
TransBack:Boolean;
BackColor:TColor;
LOFont:TFont;}
{function CreateFont(
nHeight, nWidth, nEscapement, nOrientaion, fnWeight: Integer;
fdwItalic, fdwUnderline, fdwStrikeOut, fdwCharSet,
fdwOutputPrecision,fdwClipPrecision, fdwQuality, fdwPitchAndFamily: DWORD;
lpszFace: PChar): HFONT; stdcall;}

procedure TLabelObjectsForm.PlaceThisLabel(ThisLabel: Integer);
var
  hFont {,  OldFonti}: THandle;
  OldBkMode: integer;
  alog {,FontRecord}: TLogFont;
{OldFont:TFont; }
begin {OK ... PLACE the text}
  OldBkMode := 0;

(*         GetObject(ObjectLabelArray[ThisLabel].LOFont.Handle,
         {RasterForm.Image1.Canvas.Font.Handle,}
           Sizeof(FontRecord),
           Addr(FontRecord));*)
{  FontRecord.lfEscapement:=(ObjectLabelArray[ThisLabel].RotAngle *10);}
{  FontRecord.lfOrientation:=(ObjectLabelArray[ThisLabel].RotAngle *10);}
{  RasterForm.Image1.Canvas.Font.Handle:=CreateFontIndirect(FontRecord);}
{  RasterForm.Image1.Canvas.Font:=}
{  HFont:=CreateFont(-Size, 0, Angle*10, 0, fw_normal,0,0,0,1,4,$10,2,4,PChar(FontName));}
  with alog do
  begin
//LOGFONT struct
//you can find all this in win32 help just look under LOGFONT
    lfheight :=
{      LOTEdit.Font.Size}
      {10}
    round(LOTEdit.Font.Size * LOTEdit.Font.PixelsPerInch / 72)
      ;
    lfFacename := 'ARIAL';
    lfwidth := abs(lfheight div 2);
    lfEscapement := (ObjectLabelArray[ThisLabel].RotAngle * 10);
    lfOrientation := lfescapement;
    lfweight := fw_normal;
    lfitalic := 0;
    lfunderline := 0;
    lfstrikeout := 0;
    lfcharset := ANSI_CHARSET;
    lfoutprecision := out_tt_precis;
    lfclipprecision := clip_default_precis;
    lfquality := PROOF_QUALITY;
    lfpitchandfamily := default_pitch or ff_swiss;
  end;
  hfont := createfontindirect(alog);
  if RotatedCB.Checked then begin
    if hFont <> 0 then
      with GRIPForm.Image1.Canvas do
      begin
        font.handle := hfont;
{      font.Color := clRed;}
{      font.Charset := LOTEdit.Font.Charset;}
        font.Color := LOTEdit.Font.Color;

{      font.height := LOTEdit.Font.Height;
      font.size := LOTEdit.Font.size; }
{      font.name:= LOTEdit.Font.Name;
      font.Pitch := LOTEdit.Font.Pitch;   }

{      font.Style:=LOTEdit.Font.Style;  }
{ ObjectLabelArray[ThisLabel].LOFont.Charset;
      font.Color := ObjectLabelArray[ThisLabel].LOFont.Color;
      font.height := ObjectLabelArray[ThisLabel].LOFont.Height;
      font.name:= ObjectLabelArray[ThisLabel].LOFont.Name;
      font.Pitch := ObjectLabelArray[ThisLabel].LOFont.Pitch;
      font.size := ObjectLabelArray[ThisLabel].LOFont.size;
      font.Style:=ObjectLabelArray[ThisLabel].LOFont.Style;}
      end else DoMessages(26);
  end else
    with GRIPForm.Image1.Canvas do
    begin
{if RasterForm.Image1.Canvas.Font = nil then
RasterForm.Image1.Canvas.Font:=LOTEdit.Font;}
{  OldFont:=RasterForm.Image1.Canvas.Font;}
      GRIPForm.Image1.Canvas.Font :=
        ObjectLabelArray[ThisLabel].LOFont;
    end;
{  OldFonti:=SelectObject(RasterForm.Image1.Canvas.Font.Handle, HFont);}
  if ObjectLabelArray[ThisLabel].TransBack then
    OldBkMode :=
      SetBkMode(GRIPForm.Image1.Canvas.Handle, 0 {TRANSPARENT})
  else SetBkColor(GRIPForm.Image1.Canvas.Handle,
      ObjectLabelArray[ThisLabel].BackColor);


  GRIPForm.Image1.Canvas.TextOut(
    ObjectLabelArray[ThisLabel].X,
    ObjectLabelArray[ThisLabel].Y,
    ObjectLabelArray[ThisLabel].LOText);
{  SelectObject(RasterForm.Image1.Canvas.Font.Handle, OldFonti);
  DeleteObject(HFont);}
{  RasterForm.Image1.Canvas.Font:=OldFont;}
  if ObjectLabelArray[ThisLabel].TransBack then
    SetBkMode(GRIPForm.Image1.Canvas.Handle,
      OldBkMode);

end;





end.
